@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 #include <kapplication.h>
@@ -29,6 +29,11 @@
 #include <kglobal.h>
 #include <kpushbutton.h>
 #include <kfiledialog.h>
+#include <kactionclasses.h>
+#include <ktempfile.h>
+#include <ksavefile.h>
+#include <kstandarddirs.h>
+#include <kio/netaccess.h>
 
 #include <qevent.h>
 #include <qlineedit.h>
@@ -46,6 +51,7 @@
 #include "sessionwidget.h"
 #include "tablelinkstatus.h"
 #include "treeview.h"
+#include "documentrootdialog.h"
 #include "klshistorycombo.h"
 #include "klsconfig.h"
 #include "resultview.h"
@@ -53,13 +59,22 @@
 #include "../engine/linkstatus.h"
 #include "../engine/linkchecker.h"
 #include "../engine/searchmanager.h"
+#include "resultssearchbar.h"
+#include "../actionmanager.h"
+#include "../utils/utils.h"
+#include "../utils/xsl.h"
 
 
 SessionWidget::SessionWidget(int max_simultaneous_connections, int time_out,
                              QWidget* parent, const char* name, WFlags f)
-        : SessionWidgetBase(parent, name, f), search_manager_(0),
-        ready_(true), bottom_status_timer_(this, "bottom_status_timer"),
-        max_simultaneous_connections_(max_simultaneous_connections), time_out_(time_out)
+    : SessionWidgetBase(parent, name, f), search_manager_(0), 
+        action_manager_(ActionManager::getInstance()), 
+        ready_(true), to_start_(false), to_pause_(false), to_stop_(false),
+        in_progress_(false), paused_(false), stopped_(true),
+        bottom_status_timer_(this, "bottom_status_timer"),
+        max_simultaneous_connections_(max_simultaneous_connections),
+        time_out_(time_out), tree_display_(false), follow_last_link_checked_(KLSConfig::followLastLinkChecked()), 
+        start_search_action_(0)
 {
     newSearchManager();
 
@@ -68,10 +83,10 @@ SessionWidget::SessionWidget(int max_simultaneous_connections, int time_out,
 
     connect(combobox_url, SIGNAL( textChanged ( const QString & ) ),
             this, SLOT( slotEnableCheckButton( const QString & ) ) );
-    
+
     connect(tree_view, SIGNAL( clicked ( QListViewItem * ) ),
             this, SLOT( showBottomStatusLabel( QListViewItem * ) ) );
-    
+
     connect(&bottom_status_timer_, SIGNAL(timeout()), this, SLOT(clearBottomStatusLabel()) );
 }
 
@@ -83,6 +98,25 @@ SessionWidget::~SessionWidget()
         saveCurrentCheckSettings();
 }
 
+void SessionWidget::init()
+{
+    combobox_url->init();
+
+    toolButton_clear_combo->setIconSet(SmallIconSet("locationbar_erase"));
+
+    pushbutton_url->setIconSet(KGlobal::iconLoader()->loadIconSet("fileopen", KIcon::Small));
+    QPixmap pixMap = KGlobal::iconLoader()->loadIcon("fileopen", KIcon::Small);
+    pushbutton_url->setFixedSize(pixMap.width() + 8, pixMap.height() + 8);
+    connect(pushbutton_url, SIGNAL(clicked()), this, SLOT(slotChooseUrlDialog()));
+    
+    resultsSearchBar->hide();
+
+    start_search_action_ = static_cast<KToggleAction*> (action_manager_->action("start_search"));
+    
+    connect(resultsSearchBar, SIGNAL(signalSearch(LinkMatcher)), 
+            this, SLOT(slotApplyFilter(LinkMatcher)));
+}
+
 void SessionWidget::slotLoadSettings(bool modify_current_widget_settings)
 {
     if(modify_current_widget_settings)
@@ -91,12 +125,12 @@ void SessionWidget::slotLoadSettings(bool modify_current_widget_settings)
         spinbox_depth->setValue(KLSConfig::depth());
         checkbox_subdirs_only->setChecked(!KLSConfig::checkParentFolders());
         checkbox_external_links->setChecked(KLSConfig::checkExternalLinks());
-        tree_view->setRootIsDecorated(tree_display_);
         tree_display_ = KLSConfig::displayTreeView();
+        tree_view->setTreeDisplay(tree_display_);
     }
 
     search_manager_->setTimeOut(KLSConfig::timeOut());
-    
+
     //kdDebug(23100) << "tree_display_: " << tree_display_ << endl;
 }
 
@@ -138,7 +172,6 @@ void SessionWidget::newSearchManager()
 
 void SessionWidget::setColumns(QStringList const& colunas)
 {
-    //table_linkstatus->setColumns(colunas);
     tree_view->setColumns(colunas);
 }
 
@@ -148,40 +181,8 @@ void SessionWidget::setUrl(KURL const& url)
     combobox_url->setFocus();
 }
 
-void SessionWidget::displayAllLinks()
-{
-    //table_linkstatus->showAll();
-    tree_view->showAll();
-}
-
-void SessionWidget::displayGoodLinks()
-{
-    //table_linkstatus->show(ResultView::good);
-    tree_view->show(ResultView::good);
-}
-
-void SessionWidget::displayBadLinks()
-{
-    //table_linkstatus->show(ResultView::bad);
-    tree_view->show(ResultView::bad);
-}
-
-void SessionWidget::displayMalformedLinks()
-{
-    //table_linkstatus->show(ResultView::malformed);
-    tree_view->show(ResultView::malformed);
-}
-
-void SessionWidget::displayUndeterminedLinks()
-{
-    //table_linkstatus->show(ResultView::undetermined);
-    tree_view->show(ResultView::undetermined);
-}
-
 bool SessionWidget::isEmpty() const
 {
-    //Q_ASSERT(table_linkstatus);
-    //return table_linkstatus->isEmpty();
     Q_ASSERT(tree_view);
     return tree_view->isEmpty();
 }
@@ -193,39 +194,40 @@ SearchManager const* SessionWidget::getSearchManager() const
 
 void SessionWidget::slotEnableCheckButton(const QString & s)
 {
+    if(!(stopped_ && !pendingActions()))
+        return;
+
     if(!s.isEmpty() && !search_manager_->searching())
-        pushbutton_check->setEnabled(true);
-    else
-        pushbutton_check->setEnabled(false);
-}
-/*
-void SessionWidget::slotSuggestDomain(bool toogle)
-{
-    if(toogle && !(combobox_url->currentText().isEmpty()))
     {
-        KURL url = Url::normalizeUrl(combobox_url->currentText());
-        if(url.isValid())
-            lineedit_domain->setText(url.host() + url.directory(true, false));
+        start_search_action_->setEnabled(true);
+    }
+    else
+    {
+        start_search_action_->setEnabled(false);
     }
 }
-*/
+
 void SessionWidget::slotCheck()
 {
-    if(!ready_)
-    {
-        KApplication::beep ();
-        return;
-    }
+    Q_ASSERT(to_start_);
+    Q_ASSERT(!in_progress_);
+    Q_ASSERT(!paused_);
+    Q_ASSERT(stopped_);
 
     ready_ = false;
     if(!validFields())
     {
         ready_ = true;
-        KApplication::beep ();
+        KApplication::beep();
         return;
     }
-    
+
     emit signalSearchStarted();
+    
+    in_progress_ = true;
+    paused_ = false;
+    stopped_ = false;
+    
     slotLoadSettings(false); // it seems that KConfigDialogManager is not trigering this slot
 
     newSearchManager();
@@ -238,32 +240,29 @@ void SessionWidget::slotCheck()
     progressbar_checker->setProgress(0);
     textlabel_progressbar->setText(i18n( "Checking..." ));
 
-    //table_linkstatus->verticalHeader()->show();
-    //table_linkstatus->verticalHeader()->adjustHeaderSize();
-    //table_linkstatus->setLeftMargin(table_linkstatus->verticalHeader()->width());
-
-    //buttongroup_search->setEnabled(false);
-    pushbutton_check->setEnabled(false);
-    pushbutton_cancel->setEnabled(true);
-    pushbutton_cancel->setText(i18n( "&Pause" ));
-    pushbutton_cancel->setIconSet(SmallIconSet("player_pause"));
     textlabel_elapsed_time->setEnabled(true);
     //textlabel_elapsed_time_value->setText("");
     textlabel_elapsed_time_value->setEnabled(true);
 
-    Q_ASSERT(!pushbutton_check->isEnabled()); // FIXME pushbutton_check sometimes doesn't show disable. Qt bug?
-
     //table_linkstatus->clear();
     tree_view->clear();
-    
+
     KURL url = Url::normalizeUrl(combobox_url->currentText());
+    
+    if(!url.protocol().startsWith("http")) 
+    {
+        DocumentRootDialog dialog(this, url.directory());
+        dialog.exec();
+        search_manager_->setDocumentRoot(KURL::fromPathOrURL(dialog.url()));
+    }
+    
     if(KLSConfig::useQuantaUrlPreviewPrefix() && Global::isKLinkStatusEmbeddedInQuanta())
     {
         KURL url_aux = Global::urlWithQuantaPreviewPrefix(url);
         if(url_aux.isValid() && !url_aux.isEmpty())
             url = url_aux;
     }
-    
+
     if(!checkbox_recursively->isChecked())
     {
         search_manager_->setSearchMode(SearchManager::depth);
@@ -309,8 +308,8 @@ void SessionWidget::slotCheck()
     }
     if(!lineedit_reg_exp->text().isEmpty())
     {
-    	search_manager_->setCheckRegularExpressions(true);
-	search_manager_->setRegularExpression(lineedit_reg_exp->text(), false);
+        search_manager_->setCheckRegularExpressions(true);
+        search_manager_->setRegularExpression(lineedit_reg_exp->text(), false);
     }
 
     kdDebug(23100) <<  "URI: " << url.prettyURL() << endl;
@@ -319,49 +318,22 @@ void SessionWidget::slotCheck()
     slotSetTimeElapsed();
 }
 
-void SessionWidget::slotCancel()
-{
-    if(search_manager_->searching())
-    {
-        Q_ASSERT(!ready_);
-        pushbutton_cancel->setEnabled(false);
-        search_manager_->cancelSearch();
-    }
-    else
-    {
-        Q_ASSERT(ready_);
-        Q_ASSERT(pushbutton_cancel->text() == i18n( "&Resume" ));
-        pushbutton_check->setEnabled(false);
-        pushbutton_cancel->setText(i18n( "&Pause" ));
-        pushbutton_cancel->setIconSet(SmallIconSet("player_pause"));
-        textlabel_progressbar->setText(i18n( "Checking..." ));
-        ready_ = false;
-        search_manager_->resume();
-        
-        displayAllLinks();
-        emit signalSearchStarted();
-        slotLoadSettings(isEmpty()); // it seems that KConfigDialogManager is not trigering this slot
-    }
-}
-
 void SessionWidget::keyPressEvent ( QKeyEvent* e )
 {
     if( e->key() == Qt::Key_Return &&
-            ( combobox_url->hasFocus() ||
+        ( combobox_url->hasFocus() ||
               //lineedit_domain->hasFocus() ||
               //checkbox_depth->hasFocus()  ||
-              spinbox_depth->hasFocus()  ||
+        spinbox_depth->hasFocus()  ||
               //checkbox_domain->hasFocus()  ||
               //spinbox_external_domain->hasFocus()
-              checkbox_recursively->hasFocus() ||
-              checkbox_external_links->hasFocus() ||
-              checkbox_subdirs_only->hasFocus() ) )
+        checkbox_recursively->hasFocus() ||
+        checkbox_external_links->hasFocus() ||
+        checkbox_subdirs_only->hasFocus() ) )
     {
         if(validFields())
         {
-            pushbutton_check->toggle();
-            //pushbutton_check->setEnabled(false);
-            slotCheck();
+            slotStartSearch();
         }
     }
 
@@ -388,19 +360,23 @@ bool SessionWidget::validFields()
 void SessionWidget::slotRootChecked(LinkStatus const* linkstatus, LinkChecker * anal)
 {
     slotSetTimeElapsed();
-    emit signalUpdateTabLabel(search_manager_->linkStatusRoot());
+    emit signalUpdateTabLabel(search_manager_->linkStatusRoot(), this);
+
+    Q_ASSERT(textlabel_progressbar->text() == i18n("Checking...") ||
+            textlabel_progressbar->text() == i18n("Stopped"));
 
-    Q_ASSERT(textlabel_progressbar->text() == i18n("Checking...") || 
-                textlabel_progressbar->text() == i18n("Stopped"));
     progressbar_checker->setProgress(1);
 
     //table_linkstatus->insertResult(linkstatus);
-    TreeViewItem* tree_view_item = new TreeViewItem(tree_view, tree_view->lastItem(), linkstatus, 3);
+    TreeViewItem* tree_view_item = new TreeViewItem(tree_view, tree_view->lastItem(), linkstatus);
     LinkStatus* ls = const_cast<LinkStatus*> (linkstatus);
     ls->setTreeViewItem(tree_view_item);
 
     if(linkstatus->isRedirection() && linkstatus->redirection())
         slotLinkChecked(linkstatus->redirection(), anal);
+
+    resultsSearchBar->show();
+    ActionManager::getInstance()->action("file_export_html")->setEnabled(!isEmpty());
 }
 
 void SessionWidget::slotLinkChecked(LinkStatus const* linkstatus, LinkChecker * anal)
@@ -408,62 +384,53 @@ void SessionWidget::slotLinkChecked(LinkStatus const* linkstatus, LinkChecker *
     slotSetTimeElapsed();
 
     kdDebug(23100) << textlabel_progressbar->text() << endl;
-    Q_ASSERT(textlabel_progressbar->text() == i18n("Checking...") || 
-                textlabel_progressbar->text() == i18n("Stopped"));
+
+    Q_ASSERT(textlabel_progressbar->text() == i18n("Checking...") ||
+            textlabel_progressbar->text() == i18n("Stopped"));
+
     progressbar_checker->setProgress(progressbar_checker->progress() + 1);
 
     if(linkstatus->checked())
     {
         TreeViewItem* tree_view_item = 0;
-        
+        TreeViewItem* parent_item = linkstatus->parent()->treeViewItem();
+        bool match = resultsSearchBar->currentLinkMatcher().matches(*linkstatus);
+
         if(tree_display_)
         {
             //kdDebug(23100) << "TREE!!!!!" << endl;
-            TreeViewItem* parent_item = linkstatus->parent()->treeViewItem();
-            tree_view_item = new TreeViewItem(parent_item, parent_item->lastChild(), linkstatus, 3);
+            tree_view_item = new TreeViewItem(tree_view, parent_item, parent_item->lastChild(), linkstatus);
+
             parent_item->setLastChild(tree_view_item);
-            if(KLSConfig::followLastLinkChecked())
+            if(follow_last_link_checked_)
                 tree_view->ensureRowVisible(tree_view_item, tree_display_);
-            else
-                tree_view->ensureRowVisible(tree_view->lastItem(), tree_display_);
+            
+            tree_view_item->setEnabled(match);
         }
         else
         {
             //kdDebug(23100) << "FLAT!!!!!" << endl;
-            tree_view_item = new TreeViewItem(tree_view, tree_view->lastItem(), linkstatus, 3);
-            tree_view->ensureRowVisible(tree_view_item, tree_display_);
-        }   
+            tree_view_item = new TreeViewItem(tree_view, tree_view->lastItem(), linkstatus);
+            if(follow_last_link_checked_)
+                tree_view->ensureRowVisible(tree_view_item, tree_display_);
+        
+            tree_view_item->setVisible(match);
+        }
+        
         LinkStatus* ls = const_cast<LinkStatus*> (linkstatus);
         ls->setTreeViewItem(tree_view_item);
-        
+
         if(linkstatus->isRedirection() && linkstatus->redirection())
             slotLinkChecked(linkstatus->redirection(), anal);
     }
 }
-/*
-vector<TableItem*> SessionWidget::generateRowOfTableItems(LinkStatus const* linkstatus) const
-{
-    vector<TableItem*> items;
-    int column = 1;
 
-    TableItem* item1 = new TableItemStatus(table_linkstatus, QTableItem::Never,
-                                           linkstatus, column++);
-    TableItem* item2 = new TableItemNome(table_linkstatus, QTableItem::Never,
-                                         linkstatus, column++);
-    TableItem* item3 = new TableItemURL(table_linkstatus, QTableItem::Never,
-                                        linkstatus, column++);
-    items.push_back(item1);
-    items.push_back(item2);
-    items.push_back(item3);
-
-    // If more columns are choosed in the settings, create and add the items here
-    // ...
-
-    return items;
-}
-*/
 void SessionWidget::slotSearchFinished()
 {
+    Q_ASSERT(in_progress_);
+    Q_ASSERT(!paused_);
+    Q_ASSERT(!stopped_);
+    
     KApplication::beep ();
 
     textlabel_progressbar->setText(i18n( "Ready" ));
@@ -473,32 +440,53 @@ void SessionWidget::slotSearchFinished()
     progressbar_checker->setProgress(0);
 
     ready_ = true;
-    pushbutton_check->setEnabled(true);
-    pushbutton_cancel->setEnabled(false);
-    //buttongroup_search->setEnabled(true);
+    
     textlabel_elapsed_time->setEnabled(true);
     textlabel_elapsed_time_value->setEnabled(true);
     textlabel_elapsed_time_value->setText(search_manager_->timeElapsed().toString("hh:mm:ss"));
 
+    in_progress_ = false;
+    paused_ = false;
+    stopped_ = true;
+    resetPendingActions();
+    action_manager_->slotUpdateSessionWidgetActions(this);
+    
     emit signalSearchFinnished();
 }
 
 void SessionWidget::slotSearchPaused()
 {
-    KApplication::beep ();
+    Q_ASSERT(pendingActions());
+    Q_ASSERT(in_progress_);
+    
+    KApplication::beep();
 
-    textlabel_progressbar->setText(i18n( "Stopped" ));
+    textlabel_progressbar->setText(i18n("Stopped"));
 
     ready_ = true;
-    pushbutton_check->setEnabled(true);
-    pushbutton_cancel->setEnabled(true);
-    pushbutton_cancel->setText(i18n( "&Resume" ));
-    pushbutton_cancel->setIconSet(SmallIconSet("player_play"));
+
+    if(to_stop_)
+    {
+        in_progress_ = false;
+        paused_ = false;
+        stopped_ = true;
+    }
+    else
+    {
+        Q_ASSERT(to_pause_);
+        Q_ASSERT(!stopped_);
+        
+        paused_ = true;
+    }
+    
     textlabel_elapsed_time->setEnabled(true);
     textlabel_elapsed_time_value->setEnabled(true);
     textlabel_elapsed_time_value->setText(search_manager_->timeElapsed().toString("hh:mm:ss"));
 
-    emit signalSearchFinnished();
+    resetPendingActions();
+    action_manager_->slotUpdateSessionWidgetActions(this);
+
+    emit signalSearchPaused();
 }
 
 void SessionWidget::insertUrlAtCombobox(QString const& url)
@@ -509,19 +497,17 @@ void SessionWidget::insertUrlAtCombobox(QString const& url)
 void SessionWidget::showBottomStatusLabel(QListViewItem * item)
 {
     kdDebug(23100) << "SessionWidget::showBottomStatusLabel" << endl;
-    
+
     TreeViewItem* _item = tree_view->myItem(item);
     if(_item)
     {
-        QString status = _item->linkStatus()->status();
+        QString status = _item->linkStatus()->statusText();
         textlabel_status->setText(status);
 
         if(textlabel_status->sizeHint().width() > textlabel_status->maximumWidth())
-            QToolTip::add
-                    (textlabel_status, status);
+            QToolTip::add(textlabel_status, status);
         else
-            QToolTip::remove
-                    (textlabel_status);
+            QToolTip::remove(textlabel_status);
 
         bottom_status_timer_.stop();
         bottom_status_timer_.start(5 * 1000, true);
@@ -560,21 +546,6 @@ void SessionWidget::slotLinksToCheckTotalSteps(uint steps)
     progressbar_checker->setProgress(0);
 }
 
-void SessionWidget::init()
-{
-    combobox_url->init();
-
-    pushbutton_check->setIconSet(SmallIconSet("viewmag"));
-    pushbutton_cancel->setIconSet(SmallIconSet("player_pause"));
-    toolButton_clear_combo->setIconSet(SmallIconSet("locationbar_erase"));
-	
-	pushbutton_url->setIconSet(KGlobal::iconLoader()->loadIconSet("fileopen", KIcon::Small));
-	QPixmap pixMap = KGlobal::iconLoader()->loadIcon("fileopen", KIcon::Small);
-	pushbutton_url->setFixedSize(pixMap.width() + 8, pixMap.height() + 8);
-	connect(pushbutton_url, SIGNAL(clicked()), this, SLOT(slotChooseUrlDialog()));
-
-}
-
 void SessionWidget::slotClearComboUrl()
 {
     combobox_url->setCurrentText("");
@@ -582,7 +553,168 @@ void SessionWidget::slotClearComboUrl()
 
 void SessionWidget::slotChooseUrlDialog()
 {
-	setUrl(KFileDialog::getOpenURL());
+    setUrl(KFileDialog::getOpenURL());
+}
+
+void SessionWidget::slotHideSearchPanel()
+{
+    if(buttongroup_search->isHidden())
+        buttongroup_search->show();
+    else
+        buttongroup_search->hide();
+}
+
+void SessionWidget::setFollowLastLinkChecked(bool follow)
+{
+    kdDebug(23100) << "setFollowLastLinkChecked: " << follow << endl;
+    follow_last_link_checked_ = follow;
+}
+
+void SessionWidget::slotFollowLastLinkChecked()
+{
+    follow_last_link_checked_ = !follow_last_link_checked_;
+}
+
+void SessionWidget::slotResetSearchOptions()
+{
+    slotLoadSettings(true);
+
+    combobox_url->clear();
+    lineedit_reg_exp->clear();
+}
+
+void SessionWidget::slotStartSearch()
+{
+    if(in_progress_)
+    {
+        start_search_action_->setChecked(true); // do not toggle
+        Q_ASSERT(!stopped_);
+        KApplication::beep();
+        return;
+    }
+    
+    to_start_ = true;
+    slotLoadSettings(false);
+    slotCheck();
+    resetPendingActions();
+
+    action_manager_->slotUpdateSessionWidgetActions(this);
+}
+
+void SessionWidget::slotPauseSearch()
+{
+    Q_ASSERT(in_progress_);
+    Q_ASSERT(!stopped_);
+    
+    if(pendingActions())
+        return;
+
+    to_pause_ = true;
+    
+    if(!paused_)
+    {
+        Q_ASSERT(!ready_);
+        Q_ASSERT(search_manager_->searching());
+
+        search_manager_->cancelSearch();        
+    }
+    else
+    {
+        Q_ASSERT(ready_);
+        
+        paused_ = false;
+
+        textlabel_progressbar->setText(i18n("Checking..."));
+        ready_ = false;
+        search_manager_->resume();
+
+        emit signalSearchStarted();
+        slotLoadSettings(isEmpty()); // it seems that KConfigDialogManager is not trigering this slot
+        
+        resetPendingActions();
+    }
+}
+
+void SessionWidget::slotStopSearch()
+{
+    Q_ASSERT(in_progress_);
+    Q_ASSERT(!stopped_);
+    
+    if(pendingActions())
+        return;
+
+    to_stop_ = true;
+    
+    if(!paused_)
+    {        
+        Q_ASSERT(!ready_);
+        Q_ASSERT(search_manager_->searching());
+
+        search_manager_->cancelSearch();        
+    }
+    else
+    {
+        in_progress_ = false;
+        paused_ = false;
+        stopped_ = true;
+    
+        action_manager_->slotUpdateSessionWidgetActions(this);
+    }
+}
+
+bool SessionWidget::pendingActions() const
+{
+    return (to_start_ || to_pause_ || to_stop_);
+}
+
+void SessionWidget::resetPendingActions()
+{
+    to_start_ = false;
+    to_pause_ = false;
+    to_stop_ = false;
+}
+
+void SessionWidget::slotApplyFilter(LinkMatcher link_matcher)
+{
+    tree_view->show(link_matcher);
+}
+
+void SessionWidget::slotExportAsHTML( )
+{
+    KURL url = KFileDialog::getSaveURL(QString::null,"text/html", 0, i18n("Export Results as HTML"));
+
+    if(url.isEmpty())
+        return;
+
+    QString filename;
+    KTempFile tmp; // ### only used for network export
+
+    if(url.isLocalFile())
+        filename = url.path();
+    else
+        filename = tmp.name();
+
+    KSaveFile *savefile = new KSaveFile(filename);
+    if(savefile->status() == 0) // ok
+    {
+        QTextStream *outputStream = savefile->textStream();
+        outputStream->setEncoding(QTextStream::UnicodeUTF8);
+
+        QString xslt_doc = FileManager::read(locate("appdata", "styles/results_stylesheet.xsl"));
+        XSLT xslt(xslt_doc);        
+//         kdDebug(23100) << search_manager_->toXML() << endl;
+        QString html_ouptut = xslt.transform(search_manager_->toXML());
+        (*outputStream) << html_ouptut << endl;
+
+        savefile->close();
+    }
+        
+    delete savefile;
+
+    if (url.isLocalFile())
+        return;
+
+    KIO::NetAccess::upload(filename, url, 0);
 }
 
 
