@@ -29,6 +29,27 @@ public:
   QExtFileInfo() {};
   ~QExtFileInfo() {};
 
+  /**
+   * Works like QDir::canonicalPath, but it does not return an empty
+   * string if the path does not exists, but tries to find if there
+   * is a part of the path which exists and resolv the symlinks for
+   * that part. The ending slash is kept in the result.
+   * Example:
+   * /home/user/foo points to /mnt/foo
+   * /home/user/foo/foo2 does not exists
+   * QExtFileInfo::canonicalPath("/home/user/foo/foo2/") will return
+   * /mnt/foo/foo2/ .
+   * @param path the path to resolve
+   * @return the canonical path (symlinks resolved)
+   */
+  static QString canonicalPath(const QString& path);
+
+  /**
+   * Similar to QDir::homeDirPath(), but returns a resolved path.
+   *
+   */
+  static QString homeDirPath();
+
   /** Returns the relative url of urlToConvert to baseURL. */
   static KURL toRelative(const KURL& urlToConvert, const KURL& baseURL);
   /** Convert relative url to absolute, based on baseURL. */
@@ -56,8 +77,9 @@ public:
   /** Returns the user's home directory as an url. */
   static KURL home();
   /** A slightly better working alternative of KIO::NetAccess::exists().
-      Checks for the existance of the url. */
-  static bool exists(const KURL& url, QWidget *window);
+      Checks for the existance of the url. readingOnly is true if we check if
+      the url is readable, and false if we check if it is writable.*/
+  static bool exists(const KURL& url, bool readingOnly, QWidget *window);
   /** Synchronous copy, like NetAccess::file_copy in KDE 3.2, just that it doesn't show a progress dialog */
   static bool copy( const KURL& src, const KURL& dest, int permissions=-1,
                     bool overwrite=false, bool resume=false, QWidget* window = 0L );
@@ -67,11 +89,12 @@ public:
 private:
   /** Internal methods called by the above ones. They start their own event loop and
       exit when the even loop is exited */
-  bool internalExists(const KURL& url, QWidget *window);
+  bool internalExists(const KURL& url, bool readingOnly, QWidget *window);
   bool internalCopy(const KURL& src, const KURL& target, int permissions,
                     bool overwrite, bool resume, QWidget* window);
   KURL::List allFilesInternal(const KURL& startURL, const QString& mask, QWidget *window);
   QDict<KFileItem> allFilesDetailedInternal(const KURL& startURL, const QString& mask, QWidget *window);
+  KURL::List allLocalFiles(const QString& startPath, const QString& mask);
 
   bool bJobOK;
   static QString lastErrorMsg;
