@@ -43,7 +43,7 @@
 
 
 K_EXPORT_COMPONENT_FACTORY( quantadebuggergubed,
-                            KGenericFactory<QuantaDebuggerGubed>("quantadebuggergubed"));
+                            KGenericFactory<QuantaDebuggerGubed>("quantadebuggergubed"))
 
 const char QuantaDebuggerGubed::protocolversion[] = "0.0.12";
 
@@ -375,7 +375,7 @@ void QuantaDebuggerGubed::slotReadyRead()
       m_socket->readBlock(buffer, bytes);
       buffer[bytes] = 0;
       m_buffer += buffer;
-      delete buffer;
+      delete[] buffer;
     }
 
     while(1)
@@ -620,7 +620,7 @@ bool QuantaDebuggerGubed::sendCommand(const QString& command, char * firstarg, .
   next = firstarg;
   while(next)
   {
-    ca[(QString)next] = (QString)va_arg(l_Arg, char*) ; 
+    ca[(QString)next] = (QString)va_arg(l_Arg, char*) ;
 //     kdDebug(24002) << k_lineinfo << " Added arg/valuepair " << next << ", " << ca[next].left(30) << endl;
 
     next = va_arg(l_Arg, char*);
