@@ -19,25 +19,20 @@
 
 //kde includes
 #include <kdebug.h>
-#include <kdirwatch.h>
-#include <klocale.h>
-#include <kurl.h>
 #include <ktexteditor/editinterface.h>
 
 //own includes
 #include "saparser.h"
-#include "parsercommon.h"
-#include "node.h"
-#include "qtag.h"
-#include "resource.h"
-#include "qextfileinfo.h"
+#include "sagroupparser.h"
 #include "document.h"
-#include "quantacommon.h"
 #include "dtds.h"
+#include "node.h"
+#include "parsercommon.h"
+#include "qtag.h"
+#include "quantacommon.h"
+#include "resource.h"
 
-//#undef DEBUG_PARSER
-
-extern GroupElementMapList globalGroupMap;
+//#define DEBUG_PARSER
 
 SAParser::SAParser()
 {
@@ -48,6 +43,10 @@ SAParser::SAParser()
   m_specialInsideXml = false;
   m_parsingEnabled = true;
   m_synchronous = true;
+  m_parseOneLineTimer = new QTimer(this);
+  connect(m_parseOneLineTimer, SIGNAL(timeout()), this, SLOT(slotParseOneLine()));
+  m_parseInDetailTimer = new QTimer(this);
+  connect(m_parseInDetailTimer, SIGNAL(timeout()), this, SLOT(slotParseNodeInDetail()));
 }
 
 SAParser::~SAParser()
@@ -180,7 +179,7 @@ bool SAParser::slotParseOneLine()
 #ifdef DEBUG_PARSER
                   kdDebug(24001) << "Calling slotParseOneLine from parseArea (opening group struct)." << endl;
 #endif
-                  QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
+                  m_parseOneLineTimer->start(0, true);
                 }
                 return true;
               } else  //it's a closing group structure element (like "}")
@@ -196,7 +195,7 @@ bool SAParser::slotParseOneLine()
 #ifdef DEBUG_PARSER
                     kdDebug(24001) << "Calling slotParseOneLine from parseArea (closing group struct)." << endl;
 #endif
-                    QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
+                    m_parseOneLineTimer->start(0, true);
                   }
                   return true;
                 }
@@ -210,9 +209,11 @@ bool SAParser::slotParseOneLine()
                   ParserCommon::appendAreaToTextNode(m_write, s_currentContext.area, s_currentNode);
                 else
                   s_currentNode = ParserCommon::createTextNode(m_write, s_currentNode, s_line, groupKeywordPos, s_currentContext.parentNode);
-                s_currentNode->insideSpecial = true;
-                s_currentNode->specialInsideXml = m_specialInsideXml;
-
+                if (s_currentNode)
+                {
+                  s_currentNode->insideSpecial = true;
+                  s_currentNode->specialInsideXml = m_specialInsideXml;
+                }
                 s_previousContext = s_contextStack.pop();
                 s_currentContext.parentNode = s_previousContext.parentNode;
                 s_currentContext.lastNode = s_previousContext.lastNode;
@@ -227,7 +228,7 @@ bool SAParser::slotParseOneLine()
                 tag->name = foundText;
                 tag->setStr(foundText);
                 tag->setWrite(m_write);
-                tag->setTagPosition(s_line, groupKeywordPos, s_line, s_col);
+                tag->setTagPosition(s_line, groupKeywordPos, s_line, s_col - 1);
                 tag->setDtd(s_dtd);
                 tag->type = Tag::ScriptStructureEnd;
                 tag->single = true;
@@ -254,7 +255,7 @@ bool SAParser::slotParseOneLine()
 #ifdef DEBUG_PARSER
                   kdDebug(24001) << "Calling slotParseOneLine from parseArea (group structure)." << endl;
 #endif
-                  QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
+                  m_parseOneLineTimer->start(0, true);
                 }
                 return true;
               }
@@ -307,7 +308,7 @@ bool SAParser::slotParseOneLine()
 #ifdef DEBUG_PARSER
                 kdDebug(24001) << "Calling slotParseOneLine from slotParseOneLine (nested area)." << endl;
 #endif
-                QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
+                m_parseOneLineTimer->start(0, true);
                 return true;
               }
             }
@@ -344,8 +345,11 @@ bool SAParser::slotParseOneLine()
             {
               s_currentNode = ParserCommon::createTextNode(m_write, s_currentNode, s_line, areaEndPos, s_parentNode);
             }
-            s_currentNode->insideSpecial = true;
-            s_currentNode->specialInsideXml = m_specialInsideXml;
+            if (s_currentNode)
+            {
+              s_currentNode->insideSpecial = true;
+              s_currentNode->specialInsideXml = m_specialInsideXml;
+            }
           }
           //kdDebug(24000) << QString("Special area %1 ends at %2, %3").arg(s_dtd->name).arg(s_line).arg(lastCol) << endl;
 
@@ -378,14 +382,26 @@ bool SAParser::slotParseOneLine()
             kdDebug(24001) << "Calling slotParseForScriptGroup from slotParseOneLine." << endl;
 #endif
 //            slotParseForScriptGroup();
-if (!m_synchronous)
-{
-            SAGroupParser *groupParser = new SAGroupParser(this, g_node, g_endNode, m_synchronous, m_parsingLastNode, true);
-            connect(groupParser, SIGNAL(rebuildStructureTree(bool)), SIGNAL(rebuildStructureTree(bool)));
-            connect(groupParser, SIGNAL(cleanGroups()), SIGNAL(cleanGroups()));
-            connect(groupParser, SIGNAL(parsingDone(SAGroupParser*)), SLOT(slotGroupParsingDone(SAGroupParser*)));
-            groupParser->slotParseForScriptGroup();
-}
+            if (!m_synchronous)
+            {
+              bool parsingLastNode = true;
+              Node *n = g_endNode;
+              while (n)
+              {
+                n = n->nextSibling();
+                if (n && n->insideSpecial)
+                {
+                  parsingLastNode = false;
+                  break;
+                }
+              }
+              SAGroupParser *groupParser = new SAGroupParser(this, write(), g_node, g_endNode, m_synchronous, parsingLastNode, true);
+              connect(groupParser, SIGNAL(rebuildStructureTree(bool)), SIGNAL(rebuildStructureTree(bool)));
+              connect(groupParser, SIGNAL(cleanGroups()), SIGNAL(cleanGroups()));
+              connect(groupParser, SIGNAL(parsingDone(SAGroupParser*)), SLOT(slotGroupParsingDone(SAGroupParser*)));
+              groupParser->slotParseForScriptGroup();
+              m_groupParsers.append(groupParser);
+            }
           }
 
           m_lastParsedNode = node;
@@ -466,15 +482,36 @@ if (!m_synchronous)
       }
      case QuotedString:
       {
-       //TODO: find a better, faster solution...
-        QRegExp endQuotationRx("[^\\\\]" + s_currentContext.startString +"|^" + s_currentContext.startString);
-        QString s = s_textLine.mid(s_col);
-        int pos = s.find(endQuotationRx);
+        int pos = -1;
+        int p = s_col;
+        int l = s_textLine.length();
+        while (p < l)
+        {
+          p = s_textLine.find(s_currentContext.startString, p);
+          if (p != -1)
+          {
+            if (p >= 0)
+            {
+              int i = p - 1;
+              int slahNum = 0;
+              while (i > 0 && s_textLine[i] == '\\')
+              {
+                slahNum++;
+                i--;
+              }
+              if (p == 0 || (slahNum % 2 == 0))
+              {
+                pos = p;
+                break;
+              }
+            }
+          } else
+            break;
+          p++;
+        }
         if (pos != -1)
         {
-          pos += s_col;
-          if (pos != 0)
-            pos++;
+      //    if (pos != 0) pos++;
           s_currentContext.area.eLine = s_line;
           s_currentContext.area.eCol = pos;
           //kdDebug(24000) << QString("Quoted String s_context: %1, %2, %3, %4").arg( s_currentContext.bLine).arg(s_currentContext.bCol).arg(s_currentContext.eLine).arg(s_currentContext.eCol) << endl;
@@ -510,7 +547,7 @@ if (!m_synchronous)
         int pos = s_textLine.find(s_dtd->comments[s_currentContext.startString], s_col);
         if (pos == -1 && s_dtd->comments[s_currentContext.startString] == "\n")
         {
-          int pos2 = s_textLine.find(s_areaEndString);
+          int pos2 = s_textLine.find(s_areaEndString, s_col);
           if (pos2 != -1)
           {
             pos = pos2 - 1;
@@ -575,12 +612,12 @@ if (!m_synchronous)
       //slotParseOneLine();
     }
     else
-{
+    {
 #ifdef DEBUG_PARSER
       kdDebug(24001) << "Calling slotParseOneLine from slotParseOneLine." << endl;
 #endif
-      QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
-}
+      m_parseOneLineTimer->start(0, true);
+    }
   } else
   {
     if (!m_synchronous)
@@ -645,7 +682,10 @@ Node* SAParser::parseArea(const AreaStruct &specialArea,
     else
       s_dtd = m_dtd; //fallback when there is no s_parentNode
   }
+  m_write->addDTEP(s_dtd->name);
   s_searchForSpecialAreas = (s_dtd->specialAreas.count() > 0);
+  if (s_parentNode && s_parentNode->tag->type == Tag::Comment)
+    s_searchForSpecialAreas = false;
   s_col = s_startCol + areaStartString.length();
   s_line = s_startLine;
   s_textLine = m_write->text(s_startLine, 0, s_startLine, m_write->editIf->lineLength(s_startLine));
@@ -681,7 +721,7 @@ Node* SAParser::parseArea(const AreaStruct &specialArea,
 #ifdef DEBUG_PARSER
       kdDebug(24001) << "Calling slotParseOneLine from parseArea." << endl;
 #endif
-      QTimer::singleShot(0, this, SLOT(slotParseOneLine()));
+      m_parseOneLineTimer->start(0, true);
       return 0L;
     }
   }
@@ -709,9 +749,12 @@ Node *SAParser::parsingDone()
         Node *n = m_lastParsedNode;
         if (m_useNext)
         {
+//           kdDebug(24000) << "m_lastParsedNode: " << m_lastParsedNode << " " << m_lastParsedNode->tag->tagStr() << endl;
           n->next = s_next;
           if (s_next)
+          {
             s_next->prev = n;
+          }
           n->prev = s_parentNode;
         }
         m_currentNode = n->nextSibling();
@@ -720,7 +763,7 @@ Node *SAParser::parsingDone()
 #ifdef DEBUG_PARSER
           kdDebug(24001) << "Calling slotParseNodeInDetail from parsingDone (use return values)" << endl;
 #endif
-          QTimer::singleShot(0, this, SLOT(slotParseNodeInDetail()));
+          m_parseInDetailTimer->start(0, true);
           return m_lastParsedNode;
         }
         else
@@ -730,6 +773,9 @@ Node *SAParser::parsingDone()
           kdDebug(24001) << "Emitting rebuildStructureTree from parsingDone (use return values). Enable parsing." << endl;
 #endif
           emit rebuildStructureTree(false);
+#ifdef DEBUG_PARSER
+//           kdDebug(24000) << "Calling cleanGroups from SAParser::parsingDone" << endl;
+#endif
           emit cleanGroups();
         }
     }
@@ -756,7 +802,7 @@ Node *SAParser::parsingDone()
     Node *n;
     if (s_parentNode)
     {
-      n = s_parentNode->child;
+      n = s_parentNode;//->child;
     } else
     {
       n = s_currentNode;
@@ -765,18 +811,18 @@ Node *SAParser::parsingDone()
       s_currentNode = n;
     }
     Node *g_node = n;
-    Node *g_endNode = 0L;
 #ifdef DEBUG_PARSER
     kdDebug(24001) << "Calling slotParseForScriptGroup from parsingDone. Synch:" << m_synchronous << endl;
 #endif
     //parse for groups only when doing aynchronous detailed parsing
     if (!m_synchronous)
     {
-            SAGroupParser *groupParser = new SAGroupParser(this, g_node, g_endNode, m_synchronous, m_parsingLastNode, true);
-            connect(groupParser, SIGNAL(rebuildStructureTree(bool)), SIGNAL(rebuildStructureTree(bool)));
-            connect(groupParser, SIGNAL(cleanGroups()), SIGNAL(cleanGroups()));
-            connect(groupParser, SIGNAL(parsingDone(SAGroupParser*)), SLOT(slotGroupParsingDone(SAGroupParser*)));
-            groupParser->slotParseForScriptGroup();
+      SAGroupParser *groupParser = new SAGroupParser(this, write(), g_node, 0L, m_synchronous, true /*last node*/, true);
+      connect(groupParser, SIGNAL(rebuildStructureTree(bool)), SIGNAL(rebuildStructureTree(bool)));
+      connect(groupParser, SIGNAL(cleanGroups()), SIGNAL(cleanGroups()));
+      connect(groupParser, SIGNAL(parsingDone(SAGroupParser*)), SLOT(slotGroupParsingDone(SAGroupParser*)));
+      groupParser->slotParseForScriptGroup();
+      m_groupParsers.append(groupParser);
      }
   }
 
@@ -787,6 +833,7 @@ Node *SAParser::parsingDone()
   {
       if (m_useNext && s_currentNode)
       {
+//           kdDebug(24000) << "s_currentNode: " << s_currentNode << endl;
         Node *n = s_currentNode;
         n->next = s_next;
         if (s_next)
@@ -798,7 +845,7 @@ Node *SAParser::parsingDone()
 #ifdef DEBUG_PARSER
         kdDebug(24001) << "Calling slotParseNodeInDetail from parsingDone." << endl;
 #endif
-        QTimer::singleShot(0, this, SLOT(slotParseNodeInDetail()));
+        m_parseInDetailTimer->start(0, true);
         emit rebuildStructureTree(false);
       }
       else
@@ -848,17 +895,6 @@ void SAParser::slotParseNodeInDetail()
         m_currentNode->tag->type != Tag::Text &&
         m_currentNode->tag->type != Tag::Empty)
     {
-      m_parsingLastNode = true;
-      Node *n = m_currentNode->nextSibling();
-      while (n)
-      {
-        n = n->nextSibling();
-        if (n && n->insideSpecial)
-        {
-          m_parsingLastNode = false;
-          break;
-        }
-      }
       delete m_currentNode->child;
       m_currentNode->child = 0L;
       AreaStruct area(m_currentNode->tag->area());
@@ -873,7 +909,11 @@ void SAParser::slotParseNodeInDetail()
         if (m_currentNode->next->closesPrevious)
         {
           m_currentNode->next->removeAll = false;
+          Node *secondNext = m_currentNode->next->next;
+          if (secondNext)
+            secondNext->prev = m_currentNode;
           delete m_currentNode->next;
+          m_currentNode->next = secondNext;
           m_useNext = true;
         }
       } else
@@ -903,7 +943,7 @@ void SAParser::slotParseNodeInDetail()
 #ifdef DEBUG_PARSER
 //        kdDebug(24001) << "Calling slotParseNodeInDetail from slotParseNodeInDetail." << endl; //this is really heavy debug information, enable only when really needed
 #endif
-        QTimer::singleShot(0, this, SLOT(slotParseNodeInDetail()));
+        m_parseInDetailTimer->start(0, true);
       } else
       {
 #ifdef DEBUG_PARSER
@@ -922,195 +962,24 @@ void SAParser::setParsingEnabled(bool enabled)
   kdDebug(24001) << "Parsing enabled: " << enabled << endl;
 #endif
   m_parsingEnabled = enabled;
+  if (!enabled)
+  {
+    m_parseOneLineTimer->stop();
+    m_parseInDetailTimer->stop();
+    for (QValueList<SAGroupParser*>::Iterator it = m_groupParsers.begin(); it != m_groupParsers.end(); ++it)
+    {
+      (*it)->m_parseForGroupTimer->stop();
+      delete (*it);
+    }
+    m_groupParsers.clear();
+  }
 }
 
 void SAParser::slotGroupParsingDone(SAGroupParser *groupParser)
 {
+  m_groupParsers.remove(groupParser);
   delete groupParser;
 }
 
-void SAGroupParser::slotParseForScriptGroup()
-{
-#ifdef DEBUG_PARSER
-  //kdDebug(24001) << "slotParseForScriptGroup. Synch: " << m_synchronous << endl;
-#endif
-  if (!m_parent->parsingEnabled() || (!baseNode && !m_synchronous))
-  {
-#ifdef DEBUG_PARSER
-    kdDebug(24001) << "slotParseForScriptGroup aborted. Synch: " << m_synchronous << endl;
-#endif
-    return;
-  }
-
-  if (g_node && g_node != g_endNode )
-  {
-    if (g_node->tag && (g_node->tag->type == Tag::Text || g_node->tag->type == Tag::ScriptStructureBegin))
-      parseForScriptGroup(g_node);
-    g_node = g_node->nextSibling();
-    if (m_synchronous)
-    {
-      slotParseForScriptGroup();
-      return;
-    }
-    else
-    {
-#ifdef DEBUG_PARSER
-      //  kdDebug(24001) << "Calling slotParseForScriptGroup from slotParseForScriptGroup." << endl;
-#endif
-      QTimer::singleShot(0, this, SLOT(slotParseForScriptGroup()));
-    }
-  } else
-  {
-#ifdef DEBUG_PARSER
-    kdDebug(24001) << "slotParseForScriptGroup done." << endl;
-#endif
-    if (m_lastGroupParsed && m_parsingLastNode && !m_synchronous)
-    {
-      if (m_lastGroupParsed)
-      {
-        emit cleanGroups();
-        m_lastGroupParsed = false;
-      }
-#ifdef DEBUG_PARSER
-      kdDebug(24001) << "Emitting rebuildStructureTree from slotParseForScriptGroup." << endl;
-#endif
-      emit rebuildStructureTree(true);
-    }
-  }
-}
-
-void SAGroupParser::parseForScriptGroup(Node *node)
-{
-#ifdef DEBUG_PARSER
-  QTime t;
-  t.start();
-#endif
-
-  int bl, bc, el, ec;
-  int pos;
-  QString title;
-  QString tmpStr;
-  StructTreeGroup group;
-  GroupElement groupElement;
-  GroupElementList* groupElementList;
-  KURL baseURL = QExtFileInfo::path(m_parent->write()->url());
-  QString str = node->tag->cleanStr;
-  QString tagStr = node->tag->tagStr();
-  const DTDStruct* dtd = node->tag->dtd();
-  node->tag->beginPos(bl, bc);
-  QValueList<StructTreeGroup>::ConstIterator it;
-  for (it = dtd->structTreeGroups.begin(); it != dtd->structTreeGroups.end(); ++it)
-  {
-    group = *it;
-    if (!group.hasSearchRx ||
-        node->tag->type == Tag::XmlTag ||
-        node->tag->type == Tag::XmlTagEnd ||
-        node->tag->type == Tag::Comment ||
-        node->tag->type == Tag::Empty ||
-        ( group.tagType != Tag::Text && node->tag->type != group.tagType)
-        )
-      continue;
-    pos = 0;
-    group.searchRx.setMinimal(group.isMinimalSearchRx);
-    while (pos != -1)
-    {
-      pos = group.searchRx.search(str, pos);
-      if (pos != -1) //the Node is part of this group
-      {
-        title = tagStr.mid(pos, group.searchRx.matchedLength());
-        node->tag->beginPos(bl, bc);
-        tmpStr = tagStr.left(pos);
-        int newLines = tmpStr.contains('\n');
-        bl += newLines;
-        int l = tmpStr.findRev('\n'); //the last EOL
-        bc = (l == -1) ? bc + pos : pos - l - 1;
-        newLines = title.contains('\n');
-        l = title.length();
-        el = bl + newLines;
-        ec = (newLines > 0) ? l - title.findRev('\n') : bc + l - 1;
-        pos += l;
-        AreaStruct area(bl, bc, el, ec);
-        //get the list of elements which are present in this group and
-        //have the same title. For example get the list of all group
-        //element which are variable and the matched string was "$i"
-        QString s = title;
-        groupElementList = & (globalGroupMap[group.name + "|" + s.remove(group.clearRx)]);
-        GroupElementList::Iterator elIt;
-        //Create a new tag which point to the exact location of the matched string.
-        //For example when the group defined PHP variables it
-        //points to "$i" in a node which originally contained "print $i + 1"
-        Tag *newTag = new Tag(*node->tag);
-        newTag->setTagPosition(area);
-        newTag->setStr(title);
-        newTag->name = s;
-
-        if (dtd->name == "php" && group.name == "Functions")
-        {
-          QTag *qTag = m_parent->write()->userTagList.find(s.lower());
-          if (!qTag)
-          {
-            QTag *qTag = new QTag();
-            qTag->setName(s.left(s.find('(')));
-            m_parent->write()->userTagList.insert(s.lower(), qTag);
-          }
-        }
-
-        groupElement.deleted = false;
-        groupElement.tag = newTag;
-        groupElement.node = node;
-        //Find out if the current node is inside a script structure or not.
-        //This is used to define local/global scope of the group elements.
-        Node *tmpNode = node;
-        while (tmpNode && tmpNode->tag->dtd() == dtd && tmpNode->tag->type != Tag::ScriptStructureBegin)
-        {
-          tmpNode = tmpNode->parent;
-        }
-        if (tmpNode && tmpNode->tag->type == Tag::ScriptStructureBegin)
-        {
-          groupElement.parentNode = tmpNode;
-        } else
-        {
-          groupElement.parentNode = 0L;
-        }
-        groupElement.global = true;
-        tmpNode = node->parent;
-        while (tmpNode && tmpNode->tag->dtd() == dtd)
-        {
-          if ( tmpNode->tag->type == Tag::ScriptStructureBegin && tmpNode->tag->dtd()->localScopeKeywordsRx.search(tmpNode->tag->cleanStr) != -1)
-          {
-            groupElement.global = false;
-            groupElement.parentNode = tmpNode;
-            break;
-          }
-          tmpNode = tmpNode->parent;
-        }
-        //store the pointer to the group element list where this node was put
-        //used to clear the corresponding entry from the group element lists
-        //when the node is deleted (eg. $i was deleted, so it should be deleted
-        //from the "variables | $i" group element list as well)
-        node->groupElementLists.append(groupElementList);
-        groupElementList->append(groupElement);
-        //if a filename may be present in the title, extract it
-        if (group.hasFileName && group.parseFile)
-        {
-          s.remove(group.fileNameRx);
-          KURL url;
-          QuantaCommon::setUrl(url, s.stripWhiteSpace());
-          url = QExtFileInfo::toAbsolute(url, baseURL);
-          ParserCommon::includedFiles += url.path();
-          ParserCommon::includedFilesDTD.append(dtd);
-          ParserCommon::includeWatch->addFile(url.path());
-        }
-        node->groupTag = newTag;
-
-      }
-    }
-  }
-#ifdef DEBUG_PARSER
- if (t.elapsed() > 10)
-     kdDebug(24001) << "Done: " << t.elapsed() << endl;
-#endif
-}
-
 
 #include "saparser.moc"
