@@ -49,6 +49,7 @@
 #include "wkafkapart.h"
 #include "undoredo.h"
 #include "nodeproperties.h"
+#include "tagaction.h"
 #include "tagactionset.h"
 #include "document.h"
 #include "resource.h"
@@ -84,7 +85,7 @@ public:
 
 KafkaWidget::KafkaWidget(QWidget *parent, QWidget *widgetParent, KafkaDocument *part,
                          const char *name)
-        : KHTMLPart(new KHTMLView(this, widgetParent, name), parent, name),
+        : KHTMLPart(widgetParent, name, parent, name),
         w(part)
 {
     m_contextPopupMenu = new QPopupMenu();
@@ -336,9 +337,9 @@ void KafkaWidget::keyReturn(bool specialPressed)
         //Then split if necessary the text
         if((static_cast<DOM::CharacterData>(m_currentNode)).length() == 0)
             text = m_currentNode;
-        else if(d->m_cursorOffset == 0)
+        else if(d->m_cursorOffset <= 0)
             text2 = m_currentNode;
-        else if((unsigned)d->m_cursorOffset ==
+        else if((unsigned)d->m_cursorOffset >=
                 (static_cast<DOM::CharacterData>(m_currentNode)).length())
             text = m_currentNode;
         else
@@ -593,6 +594,9 @@ bool KafkaWidget::eventFilter(QObject *, QEvent *event)
 #endif
             if(hasSelection())
                 removeSelection();
+                
+            applyQueuedToggableTagActions();                
+            
             keyReturn(keyevent->state() & ControlButton);
             d->stuckCursorHorizontalPos = false;
             break;
@@ -661,6 +665,8 @@ bool KafkaWidget::eventFilter(QObject *, QEvent *event)
                 {
                     if(hasSelection())
                         removeSelection();
+                
+                    applyQueuedToggableTagActions();                
 
                     insertText(keyevent->text(), -1);
                 }
@@ -1142,7 +1148,6 @@ void KafkaWidget::keyDelete()
         _nodeNext = getNextNode(_nodeNext, _goingTowardsRootNode);
     }
 
-#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,1)
     //If the node which is defining the cursor position has been deleted (thus changed)
     if(false && singleNodeDeleted)
     {
@@ -1189,7 +1194,6 @@ void KafkaWidget::keyDelete()
     /**m_currentNode = _node;
     setCaretPosition(m_currentNode, (long)d->m_cursorOffset);
     emit domNodeNewCursorPos(m_currentNode, d->m_cursorOffset, m_modifs);*/
-#endif
 }
 
 void KafkaWidget::keyBackspace()
@@ -1802,6 +1806,63 @@ DOM::Node KafkaWidget::getPrevNode(DOM::Node _node, bool &goingTowardsRootNode,
     return 0;
 }
 
+void KafkaWidget::updateToggableTagActions(/*const DOM::Node &domNode, long offset*/) const
+{
+//Andras: Disable toggle behavior. It is just too broken.
+return;
+
+    quantaApp->removeAllTagActionPoolItems();
+    
+    NodeSelectionInd selection;
+    selection.fillWithVPLCursorSelection();
+    
+    Node* start_node = 0, *end_node = 0;
+//     int start_offset = 0, end_offset = 0;
+        
+    start_node = kafkaCommon::getNodeFromLocation(selection.cursorNode());
+//     start_offset = selection.cursorOffset();
+    
+    if(!start_node)
+        return;
+    
+    if(selection.hasSelection())
+    {
+        end_node = kafkaCommon::getNodeFromLocation(selection.cursorNodeEndSel());
+//         end_offset = selection.cursorOffsetEndSel();
+    }
+    else
+    {
+        end_node = start_node;
+//         end_offset = start_offset;
+    }
+    
+    // Iterate all toggable toolbar actions and toggle them on or off
+    // Look if there is a selection
+    TagAction* tag_action = 0;    
+    QPtrList<TagAction> tag_actions = quantaApp->tagActions();        
+    for (tag_action = tag_actions.first(); tag_action; tag_action = tag_actions.next())
+    {
+        if(tag_action->toggable())
+        {
+            QString tag_name = tag_action->XMLTagName();
+            if(tag_name.isEmpty())
+                break;
+            
+            QDomElement data(tag_action->data());
+            QString attribute_name(data.attribute("attribute_name", QString()));
+            QString attribute_value(data.attribute("attribute_value", QString()));
+            
+            int inside_tag;
+            if(!attribute_name.isEmpty() && !attribute_value.isEmpty())
+                inside_tag = kafkaCommon::isInsideTag(start_node, end_node, tag_name, attribute_name, attribute_value);
+            else
+                inside_tag = kafkaCommon::isInsideTag(start_node, end_node, tag_name);
+            
+            tag_action->setChecked(inside_tag == 1);
+        }
+    }
+}
+
 void KafkaWidget::makeCursorVisible(int , int )
 {
     /**DOM::Range range;
@@ -2015,7 +2076,7 @@ void KafkaWidget::khtmlDrawContentsEvent(khtml::DrawContentsEvent *event)
     KHTMLPart::khtmlDrawContentsEvent(event);
 }
 
-void KafkaWidget::getCurrentNode(DOM::Node &_currentNode, int &offset)
+void KafkaWidget::getCurrentNode(DOM::Node &_currentNode, long &offset)
 {
     _currentNode = m_currentNode;
     offset = d->m_cursorOffset;
@@ -2031,6 +2092,18 @@ void KafkaWidget::setCurrentNode(DOM::Node node, int offset)
     //setCaretPosition(m_currentNode, (long)d->m_cursorOffset);
 }
 
+void KafkaWidget::setCurrentNode(Node* cursorNode, int cursorOffset)
+{
+    DOM::Node domNode;
+    long longDomNodeOffset;
+    KafkaDocument::ref()->translateNodeIntoKafkaCursorPosition(cursorNode, cursorOffset, domNode, longDomNodeOffset);
+    if (!domNode.isNull() && domNode.nodeType() != DOM::Node::TEXT_NODE &&
+         !domNode.firstChild().isNull() && domNode.firstChild().nodeType() == DOM::Node::TEXT_NODE)
+        domNode = domNode.firstChild();
+    if (!domNode.isNull())
+        setCurrentNode(domNode, (int)longDomNodeOffset);       
+}
+
 void KafkaWidget::putCursorAtFirstAvailableLocation()
 {
     kNodeAttrs *attrs = 0L;
@@ -2091,6 +2164,8 @@ void KafkaWidget::slotNewCursorPos(const DOM::Node &domNode, long offset)
 
     if(quantaApp->aTab && ViewManager::ref()->activeView()->hadLastFocus() == QuantaView::VPLFocus)
         quantaApp->aTab->setCurrentNode(w->getNode(domNode));
+
+    updateToggableTagActions(/*domNode, offset*/);            
 }
 
 void KafkaWidget::moveDomNodes(DOM::Node newParent, DOM::Node startNode, DOM::Node endNode,
@@ -2151,7 +2226,7 @@ void KafkaWidget::removeSelection()
     NodeSelectionInd selection;
     selection.fillWithVPLCursorSelection();
     Node* cursorNode = kafkaCommon::getNodeFromLocation(selection.cursorNode());
-    int cursorOffset = 0;
+    long cursorOffset = 0;
     long domNodeCursorOffset = 0;
         
     kafkaCommon::DTDRemoveSelection(selection, &cursorNode, cursorOffset, m_modifs);
@@ -2175,4 +2250,23 @@ void KafkaWidget::removeSelection()
     makeCursorVisible();
 }
 
+void KafkaWidget::applyQueuedToggableTagActions()
+{
+    QStringList queued_actions = quantaApp->tagActionPool();
+    QPtrList<TagAction> action_list = quantaApp->tagActions();
+    for(QStringList::Iterator it = queued_actions.begin(); it != queued_actions.end(); ++it) 
+    {
+        TagAction* tag_action = 0;    
+        for (tag_action = action_list.first(); tag_action; tag_action = action_list.next())
+        {
+            if(tag_action->name() == *it)
+            {
+                tag_action->slotActionActivated(KAction::EmulatedActivation, Qt::NoButton);
+                break;
+            }
+        }
+    }
+    quantaApp->removeAllTagActionPoolItems();
+}
+
 #include "kafkahtmlpart.moc"
