@@ -42,6 +42,9 @@
 #include <kurlrequester.h>
 #include <kurlrequesterdlg.h>
 
+#include <ktexteditor/markinterface.h>
+#include <ktexteditor/viewcursorinterface.h>
+
 // application headers
 #include "copyto.h"
 #include "document.h"
@@ -58,6 +61,7 @@
 #include "teammembersdlg.h"
 #include "uploadprofilespage.h"
 #include "viewmanager.h"
+#include "qpevents.h"
 
 extern QString simpleMemberStr;
 extern QString taskLeaderStr;
@@ -70,6 +74,7 @@ Project::Project(KMainWindow *parent)
   d = new ProjectPrivate(this);
   connect(d, SIGNAL(eventHappened(const QString&, const QString&, const QString& )), this, SIGNAL(eventHappened(const QString&, const QString&, const QString& )));
   d->m_mainWindow = parent;
+  d->m_uploadDialog = 0L;
   keepPasswd = true;
   d->initActions(parent->actionCollection());
 }
@@ -157,11 +162,13 @@ void Project::insertFile(const KURL& nameURL, bool repaint )
 }
 
 
-void Project::readConfig (KConfig *config)
+void Project::readConfig(KConfig *config)
 {
   d->config = config;
-
-  config->setGroup  ("Projects");
+  config->setGroup("General Options");
+  int maxRecentItems = config->readNumEntry("Recent Files Limit", 32);
+  d->m_projectRecent->setMaxItems(maxRecentItems);
+  config->setGroup ("Projects");
   d->m_projectRecent->loadEntries(config, "RecentProjects");
 }
 
@@ -169,9 +176,9 @@ void Project::readConfig (KConfig *config)
 void Project::loadLastProject(bool reload)
 {
   d->config->setGroup("Projects");
-  QStringList projectList = d->config->readPathListEntry("OpenProjects");
-  QStringList tempList = d->config->readPathListEntry("ProjectTempFiles");
-  QStringList sessionTempList = d->config->readPathListEntry("ProjectSessionTempFiles");
+  QStringList projectList = QuantaCommon::readPathListEntry(d->config, "OpenProjects");
+  QStringList tempList = QuantaCommon::readPathListEntry(d->config, "ProjectTempFiles");
+  QStringList sessionTempList = QuantaCommon::readPathListEntry(d->config, "ProjectSessionTempFiles");
   // remove all local open projects because project and temp file are the same
   for (uint i = 0; i < projectList.count(); ++i)
   {
@@ -223,12 +230,15 @@ void Project::loadLastProject(bool reload)
   d->config->writePathEntry("OpenProjects", projectList);
   d->config->writePathEntry("ProjectTempFiles", tempList);
   // now we look for the last project
-  urlPath = d->config->readPathEntry("Last Project");
+  urlPath = QuantaCommon::readPathEntry(d->config, "Last Project");
   QuantaCommon::setUrl(url, urlPath);
 
   if ( reload && (!urlPath.isEmpty() && url.isValid()))
   {
-    d->loadProject( url );
+    if (!d->loadProject(url))
+    {
+      d->config->deleteEntry("Last Project");
+    }
   }
   d->config->sync();
 }
@@ -240,10 +250,10 @@ void Project::loadLastProject(bool reload)
 
 void Project::slotOpenProject(const KURL &url)
 {
-    kdDebug(24000) << "Open recent project: " << url <<endl;
+  kdDebug(24000) << "Open recent project: " << url <<endl;
   if ( !url.isEmpty() )
   {
-    if ( !QExtFileInfo::exists(url, d->m_mainWindow) )
+    if ( !QExtFileInfo::exists(url, true, d->m_mainWindow) )
     {
       emit hideSplash();
       if (KMessageBox::questionYesNo(d->m_mainWindow,
@@ -408,13 +418,13 @@ void Project::slotRemove(const KURL& urlToRemove)
     setModified();  // there happens more than setting the flag !
   emit reloadTree( &(d->m_projectFiles), false, QStringList() );
   emit newStatus();
-  
+
   QString urlPath = QExtFileInfo::toRelative(urlToRemove, d->baseURL).path();
   QString nice = urlPath;
   nice = KStringHandler::lsqueeze(nice, 60);
   if (KMessageBox::warningContinueCancel(d->m_mainWindow, i18n("<qt>Do you want to remove <br><b>%1</b><br> from the server(s) as well?</qt>").arg(nice), i18n("Remove From Server"), KStdGuiItem::remove(), "RemoveFromServer") == KMessageBox::Continue )
   {
-    QDomNode profilesNode = d->m_sessionDom.firstChild().firstChild().namedItem("uploadprofiles");  
+    QDomNode profilesNode = d->m_sessionDom.firstChild().firstChild().namedItem("uploadprofiles");
     QDomNodeList profileList = profilesNode.toElement().elementsByTagName("profile");
     QDomElement e;
     QString s;
@@ -434,7 +444,7 @@ void Project::slotRemove(const KURL& urlToRemove)
       baseUrl.setPass(passwd);
       baseUrl.addPath(urlPath);
       KIO::NetAccess::del(baseUrl, d->m_mainWindow);
-    }  
+    }
   }
 }
 
@@ -845,21 +855,28 @@ void Project::slotUpload()
   if (! ViewManager::ref()->saveAll())
     return;
 
-  ProjectUpload *dlg = new ProjectUpload(KURL(), false, false, i18n("Upload project items..."));
-  connect(dlg, SIGNAL(eventHappened(const QString&, const QString&, const QString& )), this, SIGNAL(eventHappened(const QString&, const QString&, const QString& )));
-  dlg->show();
+  if (!d->m_uploadDialog)
+  {
+    d->m_uploadDialog = new ProjectUpload(KURL(), "", false, false, false, i18n("Upload project items..."));
+    connect(d->m_uploadDialog, SIGNAL(eventHappened(const QString&, const QString&, const QString& )), this, SIGNAL(eventHappened(const QString&, const QString&, const QString& )));
+    d->m_uploadDialog->show();
+  } else
+    d->m_uploadDialog->raise();
 }
 
-void Project::slotUploadURL(const KURL& urlToUpload, bool quickUpload)
+void Project::slotUploadURL(const KURL& urlToUpload, const QString& profileName, bool quickUpload, bool markOnly)
 {
   if (!ViewManager::ref()->saveAll())
     return;
 
   KURL url = QExtFileInfo::toRelative( urlToUpload, d->baseURL);
-
-  ProjectUpload *dlg = new ProjectUpload(url, false, quickUpload, i18n("Upload project items..."));
-  connect(dlg, SIGNAL(eventHappened(const QString&, const QString&, const QString& )), this, SIGNAL(eventHappened(const QString&, const QString&, const QString& )));
-  dlg->show();
+  if (!d->m_uploadDialog)
+  {
+    d->m_uploadDialog = new ProjectUpload(url, profileName, false, quickUpload, markOnly, i18n("Upload project items..."));
+    connect(d->m_uploadDialog, SIGNAL(eventHappened(const QString&, const QString&, const QString& )), this, SIGNAL(eventHappened(const QString&, const QString&, const QString& )));
+    d->m_uploadDialog->show();
+  } else
+    d->m_uploadDialog->raise();
 }
 
 
@@ -903,7 +920,7 @@ bool Project::contains(const KURL& url)
 {
   if (d->m_projectFiles.isEmpty())
     d->m_projectFiles.readFromXML(d->dom, d->baseURL, d->templateURL, d->excludeRx);
-  return (d->m_projectFiles.contains(url) > 0);
+  return d->m_projectFiles.contains(url);
 }
 
 void Project::slotFileDescChanged(const KURL& url, const QString& desc)
@@ -1055,6 +1072,10 @@ bool Project::hasProject()
   return !d->projectName.isNull();
 }
 
+QString Project::projectName()
+{
+  return d->projectName;
+}
 
 const QString& Project::defaultDTD()
 {
@@ -1233,7 +1254,9 @@ bool Project::queryClose()
   {
     emit eventHappened("before_project_close", d->baseURL.url(), QString::null);
     d->config->setGroup("Projects");
-    d->config->writePathEntry("Last Project", d->projectURL.url());
+    KURL url = d->projectURL.url();
+    url.setPass("");
+    d->config->writePathEntry("Last Project", url.url());
     canExit = d->uploadProjectFile();
     if (! canExit)
     {
@@ -1241,7 +1264,20 @@ bool Project::queryClose()
           canExit = true;
     }
     if (canExit)
+    {
       emit eventHappened("after_project_close", d->baseURL.url(), QString::null);
+  // empty dom tree
+      d->dom.clear();
+      d->m_sessionDom.clear();
+      d->m_events->clear();
+      d->init();
+      newProjectLoaded(d->projectName, d->baseURL, d->templateURL);
+      reloadTree( &(d->m_projectFiles), true, QStringList());
+      d->adjustActions();
+      d->m_projectRecent->setCurrentItem(-1);
+      newStatus();
+//      kapp->processEvents(QEventLoop::ExcludeUserInput | QEventLoop::ExcludeSocketNotifiers);
+    }
   }
   return canExit;
 }
@@ -1320,5 +1356,68 @@ void Project::saveBookmarks(const KURL &url, KTextEditor::MarkInterface *markIf)
   }
 }
 
+void Project::loadCursorPosition(const KURL &url, KTextEditor::ViewCursorInterface *viewCursorIf)
+{
+  if (!viewCursorIf || !hasProject() || !contains(url))
+    return;
+  QDomNodeList nl = d->m_sessionDom.elementsByTagName("item");
+  QDomElement el;
+  KURL u = QExtFileInfo::toRelative(url, d->baseURL);
+  for ( uint i = 0; i < nl.count(); i++ )
+  {
+    el = nl.item(i).toElement();
+    if ( el.attribute("url") == QuantaCommon::qUrl(u) )
+    {
+      QString s = el.attribute("line");
+      uint line, col;
+      bool ok;
+      line = s.toUInt(&ok, 10);
+      if (ok)
+      {
+        s = el.attribute("column");
+        col = s.toUInt(&ok, 10);
+        if (ok)
+         viewCursorIf->setCursorPositionReal(line, col);
+      }
+    }
+    KURL u2 = d->baseURL;
+    QuantaCommon::setUrl(u2, el.attribute("url"));
+    if (!contains(u2))
+    {
+      el.parentNode().removeChild(el);
+    }
+  }
+}
 
+void Project::saveCursorPosition(const KURL &url, KTextEditor::ViewCursorInterface *viewCursorIf)
+{
+  if (!viewCursorIf || !hasProject() || !contains(url))
+    return;
+  QDomNodeList nl = d->m_sessionDom.elementsByTagName("item");
+  QDomElement el;
+  KURL u = QExtFileInfo::toRelative(url, d->baseURL);
+  uint line, col;
+  viewCursorIf->cursorPosition(&line, &col);
+  bool found = false;
+  for ( uint i = 0; i < nl.count(); i++ )
+  {
+    el = nl.item(i).toElement();
+    if ( el.attribute("url") == QuantaCommon::qUrl(u) )
+    {
+      el.setAttribute("line", line);
+      el.setAttribute("column", col);
+      found = true;
+      break;
+    }
+  }
+  if (!found)
+  {
+    el = d->m_sessionDom.createElement("item");
+    el.setAttribute("column", col);
+    el.setAttribute("line", line);
+    el.setAttribute("url", QuantaCommon::qUrl(u));
+    QDomNode no = d->m_sessionDom.firstChild().firstChild().namedItem("itemcursorpositions");
+    no.appendChild(el);
+  }
+}
 #include "project.moc"
