@@ -39,6 +39,7 @@
 #include <kactionclasses.h>
 #include <kdialogbase.h>
 #include <kiconloader.h>
+#include <kmdcodec.h>
 #include <kmessagebox.h>
 #include <ktempfile.h>
 #include <kdirwatch.h>
@@ -98,6 +99,7 @@ Document::Document(KTextEditor::Document *doc,
   m_dirty   = false;
   busy    = false;
   changed = false;
+  m_md5sum = "";
   m_doc = doc;
   m_view = 0L; //needed, because createView() calls processEvents() and the "this" may be deleted before m_view gets a value => crash on delete m_view;
   m_view = m_doc->createView(parent, 0L);
@@ -167,6 +169,8 @@ Document::Document(KTextEditor::Document *doc,
   selectionIf = dynamic_cast<KTextEditor::SelectionInterface *>(m_doc);
   selectionIfExt = dynamic_cast<KTextEditor::SelectionInterfaceExt *>(m_doc);
   configIf = dynamic_cast<KTextEditor::ConfigInterface*>(m_doc);
+  if (configIf)
+      configIf->readConfig();
   viewCursorIf = dynamic_cast<KTextEditor::ViewCursorInterface *>(m_view);
   codeCompletionIf = dynamic_cast<KTextEditor::CodeCompletionInterface *>(m_view);
   markIf = dynamic_cast<KTextEditor::MarkInterface *>(m_doc);
@@ -177,21 +181,22 @@ Document::Document(KTextEditor::Document *doc,
     iface->setPixmap(KTextEditor::MarkInterface::markType02, SmallIcon("debug_breakpoint"));
     iface->setDescription(KTextEditor::MarkInterface::markType02, i18n("Breakpoint"));
     iface->setPixmap(KTextEditor::MarkInterface::markType05, SmallIcon("debug_currentline"));
-  }
+    iface->setDescription(KTextEditor::MarkInterface::markType08, i18n("Annotation"));
+    iface->setPixmap(KTextEditor::MarkInterface::markType08, SmallIcon("stamp"));
 
-  // FIXME: This is allows user to set breakpoints and bookmarks by clicking or rightclicking on the icon border. However, it needs some additional code to
-  // work for breakpoints and has been disabled to prevent confusion.
-  //iface->setMarksUserChangable(KTextEditor::MarkInterface::markType01 + KTextEditor::MarkInterface::markType02);
+    // This is allows user to set breakpoints and bookmarks by clicking or rightclicking on the icon border.
+    iface->setMarksUserChangable(KTextEditor::MarkInterface::markType01 + KTextEditor::MarkInterface::markType02);
+
+  }
 
   tempFile = 0;
   m_tempFileName = QString::null;
   dtdName = Project::ref()->defaultDTD();
   reparseEnabled = true;
   repaintEnabled = true;
+  delayedTextChangedEnabled = true;
   docUndoRedo = new undoRedo(this);
 
-  //each document remember wheter it has a entry in quantarc
-  m_backupEntry = false;
   //path of the backup copy file of the document
   m_backupPathValue = QString::null;
 
@@ -211,10 +216,16 @@ Document::Document(KTextEditor::Document *doc,
   connect(m_view, SIGNAL(gotFocus(Kate::View*)), SIGNAL(editorGotFocus()));
 
   connect(fileWatcher, SIGNAL(dirty(const QString&)), SLOT(slotFileDirty(const QString&)));
+
+//   connect(m_doc, SIGNAL(marksChanged()), this, SLOT(slotMarksChanged()));
+  connect(m_doc, SIGNAL(markChanged(KTextEditor::Mark, KTextEditor::MarkInterfaceExtension::MarkChangeAction)), this, SLOT(slotMarkChanged(KTextEditor::Mark, KTextEditor::MarkInterfaceExtension::MarkChangeAction)));
+
 }
 
 Document::~Document()
 {
+  if (configIf)
+      configIf->writeConfig();
   parser->clearGroups();
   //  kdDebug(24000) << "Document::~ Document: " << this << endl;
   m_doc->closeURL(false); //TODO: Workaround for a Kate bug. Remove when KDE < 3.2.0 support is dropped.
@@ -224,7 +235,7 @@ Document::~Document()
 void Document::setUntitledUrl(const QString &url)
 {
   untitledUrl = url;
-  m_doc->openURL(KURL());
+  openURL(KURL());
 }
 
 bool Document::isUntitled()
@@ -396,9 +407,9 @@ void Document::insertFile(const KURL& url)
     fileName = url.path();
   } else
   {
-    if (!KIO::NetAccess::download(url, fileName, 0))
+    if (!KIO::NetAccess::download(url, fileName, this))
     {
-      KMessageBox::error(0, i18n("<qt>Cannot download <b>%1</b>.</qt>").arg( url.prettyURL(0, KURL::StripFileProtocol)));
+      KMessageBox::error(this, i18n("<qt>Cannot download <b>%1</b>.</qt>").arg( url.prettyURL(0, KURL::StripFileProtocol)));
       return;
     }
   }
@@ -414,8 +425,9 @@ void Document::insertFile(const KURL& url)
 }
 
 /** Inserts text at the current cursor position */
-void Document::insertText(const QString &text, bool adjustCursor, bool reparse)
+void Document::insertText(const QString &a_text, bool adjustCursor, bool reparse)
 {
+  QString text = a_text;
   if(text.isEmpty())
     return;
 
@@ -423,6 +435,37 @@ void Document::insertText(const QString &text, bool adjustCursor, bool reparse)
   unsigned int line, col;
 
   viewCursorIf->cursorPositionReal(&line, &col);
+  Node *n = parser->nodeAt(line, col, true);
+  if (n && n->tag->dtd()->family != Xml)
+  {
+    int bLine, bCol;
+    n->tag->beginPos(bLine, bCol);
+    QString s = this->text(bLine, bCol, line, col);
+    bool insideQuotes = false;
+    for (int i = 0 ; i < (int)s.length() - 1; i++)
+    {
+      if (s[i] == '"' && (i == 0 || s[i-1] != '\\'))
+        insideQuotes = !insideQuotes;
+    }
+    int eLine, eCol;
+    n->tag->endPos(eLine, eCol);
+    s = this->text(line + 1, col, eLine, eCol);
+    bool closeQuotationFound = false;
+    for (int i = 0 ; i < (int)s.length() - 1; i++)
+    {
+      if (s[i] == '"' && (i == 0 || s[i-1] != '\\'))
+      {
+        closeQuotationFound = true;
+        break;
+      }
+    }
+    if (insideQuotes && closeQuotationFound)
+    {
+      text.replace("\\\"", "\"");
+      text.replace("\"", "\\\"");
+    }
+  }
+
   editIf->insertText(line, col, text);
 
   // calculate new cursor position
@@ -505,6 +548,7 @@ void Document::insertText(const QString &text, bool adjustCursor, bool reparse)
       typingInProgress = false;
       StructTreeView::ref()->slotReparse(this, baseNode , qConfig.expandLevel);
     }
+    quantaApp->updateTreeViews();
   }
 }
 
@@ -732,16 +776,21 @@ void Document::slotDelayedShowCodeCompletion()
 */
 void Document::slotFilterCompletion( KTextEditor::CompletionEntry *completion ,QString *string )
 {
+  kdDebug(24000) << *string << endl;
+  kdDebug(24000) << completion->userdata << endl;
   int pos = completion->userdata.find("|");
   QString s = completion->userdata.left(pos);
   completion->userdata.remove(0,pos+1);
   string->remove(0, s.length());
+  kdDebug(24000) << *string << endl;
+  kdDebug(24000) << completion->userdata << endl;
   if (completion->type == "charCompletion")
   {
     *string = completion->userdata;
     uint line, col;
     viewCursorIf->cursorPositionReal(&line, &col);
     QString s2 = editIf->textLine(line).left(col);
+    kdDebug(24000) << s2 << endl;
     int pos = s2.findRev('&');
     if (pos != -1)
     {
@@ -749,6 +798,7 @@ void Document::slotFilterCompletion( KTextEditor::CompletionEntry *completion ,Q
       string->remove(s2);
     }
     string->append(";");
+    kdDebug(24000) << *string << endl;
   } else
   if ( completion->type == "attributeValue")
   {
@@ -813,20 +863,21 @@ void Document::slotCharactersInserted(int line, int column, const QString& strin
         return;
       }
     }
-    if (qConfig.replaceAccented)
+  }
+  if (qConfig.replaceAccented)
+  {
+    uint c = string[0].unicode();
+    if (c > 191)
     {
-      uint c = string[0].unicode();
-      if (c > 191)
-      {
-        m_replaceLine = line;
-        m_replaceCol = column;
-        m_replaceStr = QString("&#%1;").arg(c);
-        QTimer::singleShot(0, this, SLOT(slotReplaceChar()));
-        return;
-      }
+      m_replaceLine = line;
+      m_replaceCol = column;
+      m_replaceStr = QString("&#%1;").arg(c);
+      QTimer::singleShot(0, this, SLOT(slotReplaceChar()));
+      return;
     }
   }
 
+
  if ( (string == ">") ||
       (string == "<") )
  {
@@ -999,17 +1050,46 @@ bool Document::xmlAutoCompletion(int line, int column, const QString & string)
          }
     else if ( string[0] == qConfig.attrValueQuotation )
           {
-          //we need to find the attribute name
-          QString textLine = editIf->textLine(line).left(column-1);
-          QString attribute = textLine.mid(textLine.findRev(' ')+1);
-          showCodeCompletions( getAttributeValueCompletions(tagName, attribute) );
-          handled = true;
+            //we need to find the attribute name
+            QString textLine = editIf->textLine(line).left(column-1);
+            QString attribute = textLine.mid(textLine.findRev(' ')+1);
+            if (attribute == "style" && completionDTD->insideDTDs.contains("css"))
+            {
+              completionDTD = DTDs::ref()->find("text/css");
+              completionRequested = true;
+              return scriptAutoCompletion(line, column + 1, string);
+            }
+            showCodeCompletions( getAttributeValueCompletions(tagName, attribute) );
+            handled = true;
           }
   } // else - we are inside of a tag
   if (!handled)
   {
+    //check if we are inside a style attribute, and use css autocompletion if we are
+    QString textLine = editIf->textLine(line);
+    textLine = textLine.left(column);
+    int pos = textLine.findRev('"');
+    if (pos != -1)
+    {
+      pos = textLine.findRev(' ', pos);
+      if (pos != -1)
+      {
+        textLine = textLine.mid(pos + 1);
+        pos = textLine.find('=');
+        if (pos != -1)
+        {
+          QString attribute = textLine.left(pos);
+          if (attribute == "style" && completionDTD->insideDTDs.contains("css"))
+          {
+            completionDTD = DTDs::ref()->find("text/css");
+            completionRequested = true;
+            return scriptAutoCompletion(line, column + 1, string);
+          }
+        }
+      }
+    }
     QString s = editIf->textLine(line).left(column + 1);
-    int pos = s.findRev('&');
+    pos = s.findRev('&');
     if (pos != -1)
     {
       //complete character codes
@@ -1017,7 +1097,7 @@ bool Document::xmlAutoCompletion(int line, int column, const QString & string)
       showCodeCompletions(getCharacterCompletions(s));
       handled = true;
     }
-  } 
+  }
  return handled;
 }
 
@@ -1047,8 +1127,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
       {
         if (elementList[i]->parentNode == 0L || elementList[i]->global)
         {
-          completion.text = it.key().section('|', -1);
-          completions->append( completion );
+          completion.text = it.key().section('|', -1).stripWhiteSpace();
+          completions->append(completion);
           break;
         } else
         {
@@ -1059,8 +1139,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
           }
           if (n == elementList[i]->parentNode)
           {
-            completion.text = it.key().section('|', -1);
-            completions->append( completion );
+            completion.text = it.key().section('|', -1).stripWhiteSpace();
+            completions->append(completion);
             break;
           }
         }
@@ -1077,8 +1157,8 @@ QValueList<KTextEditor::CompletionEntry>* Document::getGroupCompletions(Node *no
     {
       if (list[i].startsWith(word) && list[i] != word)
       {
-        completion.text = list[i];
-        completions->append( completion );
+        completion.text = list[i].stripWhiteSpace();
+        completions->append(completion);
       }
     }
   }
@@ -1249,7 +1329,7 @@ QValueList<KTextEditor::CompletionEntry>* Document::getTagCompletions(int line,
       completion.text = QuantaCommon::tagCase(tagNameList[i]);
     else
       completion.text = tagNameList[i];
-    completion.text = completion.text.left(completion.text.length() - 10);
+    completion.text = completion.text.left(completion.text.length() - 10).stripWhiteSpace();
     completion.comment = comments[tagNameList[i]];
     completions->append( completion );
   }
@@ -1720,7 +1800,7 @@ bool Document::scriptAutoCompletion(int line, int column, const QString& inserte
  }
  if ( !handled && !argHintVisible &&
       (completionRequested ||
-       (s[i] == completionDTD->tagAutoCompleteAfter && (insertedString == " " || insertedString[0] == completionDTD->tagAutoCompleteAfter)) ||
+       (s[i] == completionDTD->tagAutoCompleteAfter && (insertedString == " " || (insertedString[0] == completionDTD->tagAutoCompleteAfter && !completionDTD->requestSpaceBeforeTagAutoCompletion))) ||
        completionDTD->tagAutoCompleteAfter == '\1' || (!completionDTD->memberAutoCompleteAfter.pattern().isEmpty() && completionDTD->memberAutoCompleteAfter.searchRev(s) != -1))
        )
  {
@@ -2013,10 +2093,18 @@ bool Document::xmlCodeCompletion(int line, int col)
       index = tag->valueIndexAtPos(line,col);
       if (index != -1)      //inside a value
       {
-        tag->attributeValuePos(index, bLine, bCol);
-        s = tag->attributeValue(index).left(col - bCol);
-        showCodeCompletions( getAttributeValueCompletions(tagName, tag->attribute(index), s) );
-        handled = true;
+        s = tag->attribute(index);
+        if (s == "style" && completionDTD->insideDTDs.contains("css"))
+        {
+          completionDTD = DTDs::ref()->find("text/css");
+          return scriptAutoCompletion(line, col, "");
+        } else
+        {
+          tag->attributeValuePos(index, bLine, bCol);
+          s = tag->attributeValue(index).left(col - bCol);
+          showCodeCompletions( getAttributeValueCompletions(tagName, tag->attribute(index), s) );
+          handled = true;
+        }
       } else
       {
         index = tag->attributeIndexAtPos(line,col);
@@ -2079,31 +2167,40 @@ void Document::checkDirtyStatus()
     }
     if (m_dirty)
     {
-      //check if the file is changed, also by file content. Might help to reduce
-      //unwanted warning on NFS
-      QFile f(fileName);
-      QFile tmpFile(m_tempFileName);
-      if (f.open(IO_ReadOnly) && tmpFile.open(IO_ReadOnly))
+      if (m_md5sum.isEmpty())
       {
-        QString encoding = quantaApp->defaultEncoding();
-        if (encodingIf)
-          encoding = encodingIf->encoding();
-        if (encoding.isEmpty())
-          encoding = "utf8";  //final fallback
-  
-        QString content;
-        QTextStream stream(&f);
-        stream.setCodec(QTextCodec::codecForName(encoding));
-        content = stream.read();
-        QString tmpContent;
-        QTextStream tmpStream(&tmpFile);
-        tmpStream.setCodec(QTextCodec::codecForName(encoding));
-        tmpContent = tmpStream.read();
-        if (content == tmpContent)
+        QFile f(fileName);
+        if (f.open(IO_ReadOnly))
         {
-          m_dirty = false;
+          const char* c = "";
+          KMD5 context(c);
+          context.reset();
+          context.update(f);
+          m_md5sum = context.hexDigest();
+          f.close();
+        }
+        m_dirty = false;
+      } else
+      {
+        //check if the file is changed, also by file content. Might help to reduce
+        //unwanted warning on NFS
+        QFile f(fileName);
+        if (f.open(IO_ReadOnly))
+        {
+          QString md5sum;
+          const char* c = "";
+          KMD5 context(c);
+          context.reset();
+          context.update(f);
+          md5sum = context.hexDigest();
+          kdDebug(24000) << "MD5 sum of current doc: " << m_md5sum << endl;
+          kdDebug(24000) << "MD5 sum of doc on disc : " << md5sum << endl;
+          if (md5sum == m_md5sum)
+          {
+            m_dirty = false;
+          }
+          f.close();
         }
-        f.close();
       }
     }
     if (m_dirty)
@@ -2119,7 +2216,7 @@ void Document::checkDirtyStatus()
       if (dlg->exec())
       {
           m_doc->setModified(false);
-          m_doc->openURL(url());
+          openURL(url());
       }
       m_modifTime = QFileInfo(fileName).lastModified();
       delete dlg;
@@ -2151,6 +2248,29 @@ void Document::save()
 //  kdDebug(24000) << "Document " << url() << " saved." << endl;
 }
 
+bool Document::saveAs(const KURL& url)
+{
+  bool result = m_doc->saveAs(url);
+  if (result)
+  {
+    m_md5sum = "";
+    if (url.isLocalFile())
+    {
+      QFile f(url.path());
+      if (f.open(IO_ReadOnly))
+      {
+        const char* c = "";
+        KMD5 context(c);
+        context.reset();
+        context.update(f);
+        m_md5sum = context.hexDigest();
+        f.close();
+      }
+    }
+  }
+  return result;
+}
+
 void Document::enableGroupsForDTEP(const QString& dtepName, bool enable)
 {
   if (m_groupsForDTEPs.isEmpty())
@@ -2181,11 +2301,13 @@ bool Document::evenQuotes(const QString &text)
 void Document::slotTextChanged()
 {
   changed = true;
-  if (reparseEnabled)
+  parser->setSAParserEnabled(false); //disable special area parsing if the text was changed.
+  if (reparseEnabled && delayedTextChangedEnabled)
   {
     kdDebug(24000) << "Delayed text changed called." << endl;
     //delay the handling, otherwise we may get wrong values for (line,column)
     QTimer::singleShot(0, this, SLOT(slotDelayedTextChanged()));
+    delayedTextChangedEnabled = false;
   }
 }
 
@@ -2193,10 +2315,11 @@ void Document::slotDelayedTextChanged(bool forced)
 {
    if (!forced && typingInProgress)
    {
-    // kdDebug(24000) << "Reparsing delayed!" << endl;
+     kdDebug(24000) << "Reparsing delayed!" << endl;
      parser->setParsingNeeded(true);
      QTimer::singleShot(1000, this, SLOT(slotDelayedTextChanged()));
      reparseEnabled = false;
+     delayedTextChangedEnabled = false;
      return;
    }
 
@@ -2210,7 +2333,9 @@ void Document::slotDelayedTextChanged(bool forced)
       viewCursorIf->cursorPositionReal(&line, &column);
       node = parser->nodeAt(line, column, false);
       if (node &&
-          (node->tag->type==Tag::XmlTag || node->tag->type == Tag::XmlTagEnd) )
+          ((node->tag->type == Tag::XmlTag && !node->tag->single) ||
+            node->tag->type == Tag::XmlTagEnd)
+           )
       {
         Tag *tag;
         tag = new Tag(*node->tag);
@@ -2228,7 +2353,7 @@ void Document::slotDelayedTextChanged(bool forced)
         }
       }
     }
-
+    parser->setSAParserEnabled(true); //enable special area parsing, it was disabled in slotTextChanged()
     baseNode = parser->rebuild(this);
     if (qConfig.updateClosingTags && currentNode)
     {
@@ -2236,7 +2361,7 @@ void Document::slotDelayedTextChanged(bool forced)
       node = parser->nodeAt(line, column, false);
       if (node &&
           node->tag->nameSpace + node->tag->name != currentNode->tag->nameSpace + currentNode->tag->name &&
-          (node->tag->type == Tag::XmlTag || node->tag->type == Tag::XmlTagEnd) && node->tag->validXMLTag)
+          ((node->tag->type == Tag::XmlTag && !node->tag->single) || node->tag->type == Tag::XmlTagEnd) && node->tag->validXMLTag)
       {
         int bl, bc, bl2, bc2;
         node->tag->beginPos(bl, bc);
@@ -2246,13 +2371,14 @@ void Document::slotDelayedTextChanged(bool forced)
           previousNode->tag->beginPos(bl2, bc2);
           delete currentNode;
           currentNode = previousNode;
+          previousNode = 0L;
         } else
         {
           delete previousNode;
           previousNode = 0L;
         }
         if (bl == bl2 && bc == bc2 &&
-           (currentNode->tag->type == Tag::XmlTag || currentNode->tag->type == Tag::XmlTagEnd))
+           ((node->tag->type == Tag::XmlTag && !node->tag->single) || currentNode->tag->type == Tag::XmlTagEnd))
         {
           QString newName = node->tag->name;
           bool updateClosing = (currentNode->tag->type == Tag::XmlTag) && !newName.startsWith("!");
@@ -2265,7 +2391,7 @@ void Document::slotDelayedTextChanged(bool forced)
             node = node->previousSibling();
           while (node)
           {
-            if (node->tag->validXMLTag && (node->tag->type == Tag::XmlTag || node->tag->type == Tag::XmlTagEnd))
+            if (node->tag->validXMLTag && ((node->tag->type == Tag::XmlTag && !node->tag->single) || node->tag->type == Tag::XmlTagEnd))
             {
               if (node->tag->nameSpace + node->tag->name == currentNode->tag->nameSpace + currentNode->tag->name )
               {
@@ -2313,9 +2439,10 @@ void Document::slotDelayedTextChanged(bool forced)
             else
               node = node->previousSibling();
           }
-          delete currentNode;
         }
       }
+      delete currentNode;
+      delete previousNode;
     }
 
     quantaApp->slotNewLineColumn();
@@ -2325,6 +2452,7 @@ void Document::slotDelayedTextChanged(bool forced)
       StructTreeView::ref()->slotReparse(this, baseNode , qConfig.expandLevel);
     }
     reparseEnabled = true;
+   delayedTextChangedEnabled = true;
 }
 
 /** Returns list of values for attribute */
@@ -2467,11 +2595,6 @@ void Document::clearErrorMarks()
   }
 }
 
-/** obvious */
-void Document::setBackupEntry(bool b)
-{
-   m_backupEntry = b;
-}
 QString Document::backupPathEntryValue()
 {
   return m_backupPathValue;
@@ -2485,59 +2608,58 @@ void Document::setBackupPathEntryValue(const QString& ev)
 /** if the document is modified then backup it and insert an entry in quantarc */
 void Document::createBackup(KConfig* config)
 {
-    if(isModified())
+  if (isModified())
+  {
+    m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().path());
+    if (isUntitled())
+      m_backupPathValue.append('U');
+
+    //the encoding used for the current document
+    QString encoding = quantaApp->defaultEncoding();
+    if (encodingIf)
+      encoding = encodingIf->encoding();
+    if (encoding.isEmpty())
+      encoding = "utf8";  //final fallback
+
+    //creates an entry string in quantarc if it does not exist yet
+    config->setGroup("General Options");
+    QStringList backedupFilesEntryList = QuantaCommon::readPathListEntry(config, "List of backedup files"); //the files that were backedup
+    QStringList autosavedFilesEntryList = QuantaCommon::readPathListEntry(config, "List of autosaved files"); //the list of actual backup files inside $KDEHOME/share/apps/quanta/backups
+    if (!autosavedFilesEntryList.contains(m_backupPathValue)) //not yet backed up, add an entry for this file
+    {
+      autosavedFilesEntryList.append(m_backupPathValue);
+      config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
+      backedupFilesEntryList.append(url().path() + "." + qConfig.quantaPID);
+      config->writePathEntry("List of backedup files", backedupFilesEntryList);
+      config->sync();
+    }
+
+    //creates a copy of this specific document
+    QFile file(m_backupPathValue);
+    if (file.open(IO_WriteOnly))
     {
-     if(isUntitled())
-      m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().path())+"U";
-     else
-      m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().path());
-     //creates an entry string in quantarc if it does not exist yet
-     config->setGroup("General Options");
-     QStringList backedupFilesEntryList = config->readPathListEntry("List of backedup files");
-     QStringList autosavedFilesEntryList = config->readPathListEntry("List of autosaved files");
-     QString encoding = quantaApp->defaultEncoding();
-     if (encodingIf)
-         encoding = encodingIf->encoding();
-     if (encoding.isEmpty())
-         encoding = "utf8";  //final fallback
-    if(!autosavedFilesEntryList.contains(m_backupPathValue))
-     {
-       autosavedFilesEntryList.append(m_backupPathValue);
-       config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
-       backedupFilesEntryList.append(url().path() + "." + qConfig.quantaPID);
-       config->writePathEntry("List of backedup files", backedupFilesEntryList);
-       config->sync();
-       setBackupEntry(true);
-     }
-     //creates a copy of this specific document
-     QFile file(m_backupPathValue);
-     if(file.open(IO_WriteOnly))
-     {
       QTextStream stream(&file);
       stream.setCodec(QTextCodec::codecForName(encoding));
       stream << editIf->text();
       file.close();
-     }
-
     }
+  }
 }
-/** if there is no more need of a backup copy then remove it */
+/** if there is no more need for a backup copy then remove it */
 void Document::removeBackup(KConfig *config)
 {
   config->reparseConfiguration();
   config->setGroup("General Options");
 
-  QStringList backedupFilesEntryList = config->readPathListEntry("List of backedup files");
-  QStringList autosavedFilesEntryList = config->readPathListEntry("List of autosaved files");
+  QStringList backedupFilesEntryList = QuantaCommon::readPathListEntry(config, "List of backedup files");
+  QStringList autosavedFilesEntryList = QuantaCommon::readPathListEntry(config, "List of autosaved files");
 
   autosavedFilesEntryList.remove(m_backupPathValue);
-  config->writePathEntry("List of autosaved files",autosavedFilesEntryList);
+  config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
   backedupFilesEntryList.remove(url().path() + "." + qConfig.quantaPID);
   config->writePathEntry("List of backedup files", backedupFilesEntryList);
   config->sync();
 
-  setBackupEntry(false);
-
   if(QFile::exists(m_backupPathValue))
     QFile::remove(m_backupPathValue);
 }
@@ -2681,7 +2803,7 @@ void Document::open(const KURL &url, const QString &encoding)
     }
     connect(m_doc, SIGNAL(completed()), this, SLOT(slotOpeningCompleted()));
     connect(m_doc, SIGNAL(canceled(const QString&)), this, SLOT(slotOpeningFailed(const QString&)));
-    if (!m_doc->openURL(url))
+    if (!openURL(url))
       slotOpeningFailed(QString::null);
     if (!url.isLocalFile())
     {
@@ -2714,6 +2836,7 @@ void Document::slotOpeningCompleted()
 
 void Document::slotOpeningFailed(const QString &errorMessage)
 {
+  m_md5sum = "";
   Q_UNUSED(errorMessage); //TODO: append the error message to our own error message
   if (!url().isLocalFile())
     qApp->exit_loop();
@@ -2826,12 +2949,21 @@ void Document::processDTD(const QString& documentType)
       }
    } else //DOCTYPE not found in file
    {
-     QString mimetype = KMimeType::findByURL(url())->name();
-     const DTDStruct *currdtd = DTDs::ref()->DTDfromMimeType(mimetype);
-     if (currdtd)
-        setDTDIdentifier(currdtd->name);
-     else
-        setDTDIdentifier(projectDTD);
+     KURL u = url();
+     QString dtdId = DTDs::ref()->DTDforURL(u)->name;
+//     if (dtdId == "empty")
+     {
+       const DTDStruct * dtd = DTDs::ref()->find(projectDTD);
+       if (DTDs::canHandle(dtd, u))
+        dtdId = projectDTD;
+       else
+       {
+         dtd = DTDs::ref()->find(qConfig.defaultDocType);
+         if (DTDs::canHandle(dtd, u))
+           dtdId = qConfig.defaultDocType;
+       }
+     }
+     setDTDIdentifier(dtdId);
    }
  } else //dtdName is read from the method's parameter
  {
@@ -2861,6 +2993,17 @@ void Document::slotFileDirty(const QString& fileName)
   }
 }
 
+void Document::slotMarkChanged(KTextEditor::Mark mark, KTextEditor::MarkInterfaceExtension::MarkChangeAction action)
+{
+  if(mark.type & KTextEditor::MarkInterface::markType02)
+  {
+    if(action == KTextEditor::MarkInterfaceExtension::MarkRemoved)
+      emit breakpointUnmarked(this, mark.line);
+    else
+      emit breakpointMarked(this, mark.line);
+  }
+}
+
 void Document::resetDTEPs()
 {
   m_DTEPList.clear();
@@ -2883,4 +3026,96 @@ QStringList Document::groupsForDTEPs()
     return m_groupsForDTEPs;
 }
 
+QString Document::annotationText(uint line)
+{
+  QMap<uint, QPair<QString, QString> >::Iterator it = m_annotations.find(line);
+  if (it != m_annotations.end())
+    return it.data().first;
+  else
+   return QString::null;
+}
+
+void Document::setAnnotationText(uint line, const QString& text)
+{
+  if (text.isEmpty())
+  {
+    m_annotations.remove(line);
+    if (markIf)
+      markIf->removeMark(line, KTextEditor::MarkInterface::markType08);
+  } else
+  {
+    m_annotations.insert(line, qMakePair(text, QString("")));
+    if (markIf)
+      markIf->setMark(line, KTextEditor::MarkInterface::markType08);
+    uint line, column;
+    viewCursorIf->cursorPositionReal(&line, &column);
+    viewCursorIf->setCursorPositionReal(line, 0);
+    const DTDStruct *dtd = currentDTD(true);
+    QString commentBegin = "";
+    QString commentEnd = "";
+    for (QMap<QString, QString>::ConstIterator it = dtd->comments.constBegin(); it != dtd->comments.constEnd(); ++it)
+    {
+      commentBegin = it.key();
+      commentEnd = it.data();
+      if (commentEnd != "\n")
+        break;
+    }
+    if (commentBegin.isEmpty())
+    {
+      if (dtd->family == Xml)
+      {
+        commentBegin = "<!--";
+        commentEnd = "-->";
+      } else
+      {
+        commentBegin = "/*";
+        commentEnd = "*/";
+      }
+    }
+    QString s = "@annotation: " + text;
+    s.prepend(commentBegin + " ");
+    s.append(" " + commentEnd + "\n");
+    insertText(s, true, true);
+    emit showAnnotation(line, "", qMakePair(text, QString("")));
+  }
+}
+
+void Document::addAnnotation(uint line, const QPair<QString, QString>& annotation)
+{
+  m_annotations.insert(line, annotation);
+  if (markIf)
+    markIf->setMark(line, KTextEditor::MarkInterface::markType08);
+  emit showAnnotation(line, "", annotation);
+}
+
+void Document::clearAnnotations()
+{
+  if (markIf)
+  {
+    QPtrList<KTextEditor::Mark> m = markIf->marks();
+    for (uint i=0; i < m.count(); i++)
+      markIf->removeMark( m.at(i)->line, KTextEditor::MarkInterface::markType08 );
+ }
+ m_annotations.clear();
+}
+
+bool Document::openURL(const KURL& url)
+{
+  m_md5sum = "";
+  if (url.isLocalFile())
+  {
+    QFile f(url.path());
+    if (f.open(IO_ReadOnly))
+    {
+      const char* c = "";
+      KMD5 context(c);
+      context.reset();
+      context.update(f);
+      m_md5sum = context.hexDigest();
+      f.close();
+    }
+  }
+  return m_doc->openURL(url);
+}
+
 #include "document.moc"
