@@ -19,12 +19,15 @@
 #define QUANTA_H
 
 #define QUANTA_PACKAGE "quanta"
-#define QUANTA_VERSION "3.4.3"
+#define QUANTA_VERSION "3.5.5"
 
 #define IDS_STATUS      1
 #define IDS_INS_OVR     2
 #define IDS_MODIFIED    3
 #define IDS_STATUS_CLM  4
+// Number 5 is used by the debugger, but not added until debugger is activated
+#define IDS_STATUS_DEBUGGER  5
+
 #define IDS_DEFAULT     "Ready."
 
 // include files for Qt
@@ -74,7 +77,9 @@ class KProcess;
 class KSelectAction;
 class KRecentFilesAction;
 class KToolBarPoupAction;
+class KTextEdit;
 
+class TagAction;
 class Node;
 class Parser;
 class DocTreeView;
@@ -82,6 +87,7 @@ class FilesTreeView;
 class ScriptTreeView;
 class EnhancedTagAttributeTree;
 class Project;
+class AnnotationOutput;
 class MessageOutput;
 class QDomDocument;
 class Document;
@@ -122,6 +128,7 @@ public:
 //TODO: check if we really need these "get" methods (and get rid o get)
   MessageOutput *messageOutput() const {return m_messageOutput;}
   MessageOutput *problemOutput() const {return m_problemOutput;}
+  AnnotationOutput *annotationOutput() const {return m_annotationOutput;}
 
   DebuggerManager *debugger() const {return m_debugger;}
   KParts::PartManager *partManager() {return m_partManager;}
@@ -144,12 +151,18 @@ public:
   /** Return the list of opened URLs and their editor interface numbers*/
   QStringList openedURLs() const;
   QString saveCurrentFile();
+  /**
+   * Sets the DTEP for the current document.
+   * @param dtepName the name (nickname or full name) of the DTEP
+   * @param convert  if true, converts the !DOCTYPE line to the new DTEP
+   */
+  void setDtep(const QString& dtepName, bool convert);
   QStringList tagAreas(const QString& name, bool includeCoordinates, bool skipFoundContent) const;
   QString documentFolderForURL(const QString &url);
   QString urlWithPreviewPrefix(const QString &url);
   void addFileToProject(const QString &url);
   void addFolderToProject(const QString &url);
-  /** Capture DCOP signals from KXsldbgPart or similar plugin */
+  void uploadURL(const QString &url, const QString& profile, bool markOnly);  /** Capture DCOP signals from KXsldbgPart or similar plugin */
   void newCursorPosition(const QString &file, int lineNumber, int columnNumber);
   void newDebuggerPosition(const QString &file, int lineNumber);
   void openFile(const QString &file, int lineNumber, int columnNumber);
@@ -188,12 +201,28 @@ public:
   void startIdleTimer();
 
   /** Called when a document was closed. Resets some variables. */
-  void slotFileClosed();
+  void slotFileClosed(Document *w);
   void setTabToolTip(QWidget *w, const QString& toolTipStr);
 
   void createPreviewPart();
   void createDocPart();
 
+  /**
+   * This is a pool for actions waiting for being applied (VPL only yet).
+   * For example, whee a user presses the bold action, it isn't immediately applied on the document.
+   * If the user presses a key right away the character is inserted inside the tags for the queued actions.
+   * If the user changes the place of the cursor, the actions waiting for being inserted are removed.
+   */
+  QStringList const& tagActionPool() const {return m_tagActionPool;}
+  void insertTagActionPoolItem(QString const& action_item);
+  void removeTagActionPoolItem(QString const& action_item);
+  void removeAllTagActionPoolItems() {m_tagActionPool.clear();}
+  /** Updates the structure and attribute treeview. */
+  void updateTreeViews();
+  void setTitle(const QString&);
+
+  QPtrList<TagAction> const& tagActions() const {return m_tagActions;}
+
   /** Clicked word or selected text for context sensitive menu in editor */
   QString popupWord;
 
@@ -205,9 +234,12 @@ signals: // Signals
 
   /** Emitted when some kind of event that can have associated actions has happened. */
   void eventHappened(const QString&, const QString&, const QString& );
-  
+
   void toolbarRemoved(const QString&);
 
+  void showMessage(const QString&, bool);
+  void clearMessages();
+
 public slots:
   void slotFileNew();
   void slotFileOpen();
@@ -224,7 +256,6 @@ public slots:
   void slotFileSaveAll();
   void slotFileReload(QuantaView *view = 0L);
   void slotFileReloadAll();
-  void slotFileClose();
   /** Close the document specified in the parameter if it's opened */
   void slotFileClose(const KURL &url);
   void slotFileCloseAll();
@@ -273,10 +304,12 @@ public slots:
 
   void slotShowMessagesView();
   void slotShowProblemsView();
+  void slotShowAnnotationView();
 
   void slotContextMenuAboutToShow();
 
   /** options slots */
+  void slotShowMenuBar();
   void slotPreviewOptions();
   void slotOptions();
   void slotOptionsConfigureKeys();
@@ -380,7 +413,9 @@ public slots:
 
   /** Called when the preview or documentation part is deleted. */
   void slotHTMLPartDeleted(QObject *object);
-  
+
+  void slotRefreshActiveWindow();
+
   bool slotEnableIdleTimer(bool enable);
 
 //Overridden KMdiMainFrm slots
@@ -395,7 +430,7 @@ public slots:
 
   // Get saved position of dock
   KDockWidget::DockPosition prevDockPosition(QWidget* dock, KDockWidget::DockPosition def);
-  
+
 protected slots:
   void slotDockWidgetHasUndocked(KDockWidget *widget);
   void slotPreviewBeingClosed();
@@ -485,18 +520,19 @@ protected slots:
   void slotTabDragged(QWidget *widget);
   void slotTabMoved(int from, int to);
   void slotTabAboutToMove(int from, int to);
+  void slotAnnotate();
 
 protected:
   /** Create a DTEP tarball which can be uploaded or sent in email. Returns
   *   the name of the created file or QString::null if creation has failed.
   */
   QString createDTEPTarball();
-  
+
   /** Create a toolbar tarball which can be uploaded or sent in email. Returns
   *   the name of the created file or QString::null if creation has failed.
   */
   QString createToolbarTarball();
-  
+
   /** Ask for save all the modified user toolbars. */
   bool removeToolbars();
   /** Returns true if all toolbars are hidden, false otherwise. */
@@ -511,13 +547,13 @@ protected:
   bool saveToolbar(bool localToolbar = true, const QString& toolbarToSave = QString::null, const KURL& destURL = KURL());
   /** Saves the toolbar and the actions. Returns the name of the actions file*/
   KURL saveToolbarToFile(const QString& toolbarName,const KURL& destFile);
-  void setTitle(const QString&);
-  /** Updates the structure and attribute treeview. */
-  void updateTreeViews();
   /** Makes the tabwidget look and behave like we want. If closeButtonsOnly is true,
   only the close button behavior is changed. */
   void initTabWidget(bool closeButtonsOnly = false);
 
+  void dropEvent(QDropEvent *ev);
+  void dragEnterEvent ( QDragEnterEvent * );
+
   void resetDockLayout();
 
   ScriptTreeView *scriptTab;
@@ -528,8 +564,10 @@ private:
   /** Message output window */
   MessageOutput *m_messageOutput;
   MessageOutput *m_problemOutput;
+  AnnotationOutput *m_annotationOutput;
   KMdiToolViewAccessor* m_messageOutputView;
   KMdiToolViewAccessor* m_problemsOutputView;
+  KMdiToolViewAccessor* m_annotationOutputView;
   KMdiToolViewAccessor* m_previewToolView;
   KMdiToolViewAccessor* m_documentationToolView;
   Document *m_previewedDocument;
@@ -583,6 +621,9 @@ private:
 
   QString m_scriptOutput;
 
+  QStringList m_tagActionPool;
+  QPtrList<TagAction> m_tagActions;
+
 protected: // Protected attributes
   /** Timer to refresh the structure tree. */
   QTimer *refreshTimer;
